@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -16,12 +16,16 @@ class PETSc(package.Package):
     package.Package.__init__(self,argdb,log)
     self.packagename = 'petsc'
 
+  def ShowHelp(self):
+    pass
+
   def Check(self):
-    self.havepackage = self.Link([],[],[])
+    (result, output) = self.Link([],[],[])
+    self.havepackage = result
 
   def InitDir(self,prefixdir):
     if 'PETSC_DIR' in os.environ:
-      self.dir = os.environ['PETSC_DIR']
+      self.dir = os.path.normpath(os.environ['PETSC_DIR'])
       if not os.path.exists(self.dir):
         self.log.Exit('PETSC_DIR enviroment variable is not valid')
     else:
@@ -46,6 +50,7 @@ class PETSc(package.Package):
           elif l[1] == 'PETSC_VERSION_SUBMINOR':
             subminor = l[2]
       f.close()
+      if self.release=='0': subminor = '99'
       self.version = major + '.' + minor
       self.lversion = major + '.' + minor + '.' + subminor
       self.nversion = int(major)*100 + int(minor)
@@ -58,9 +63,9 @@ class PETSc(package.Package):
       (status, output) = self.RunCommand('cd '+self.dir+';git rev-parse')
       if not status:
         self.isrepo = True
-        (status, self.gitrev) = self.RunCommand('cd '+self.dir+';git log -1 --pretty=format:%H')
+        (status, self.gitrev)  = self.RunCommand('cd '+self.dir+';git log -1 --pretty=format:%H')
         (status, self.gitdate) = self.RunCommand('cd '+self.dir+';git log -1 --pretty=format:%ci')
-        (status, self.branch) = self.RunCommand('cd '+self.dir+';git describe --contains --all HEAD')
+        (status, self.branch)  = self.RunCommand('cd '+self.dir+';git describe --contains --all HEAD')
 
   def LoadConf(self):
     if 'PETSC_ARCH' in os.environ and os.environ['PETSC_ARCH']:
@@ -86,6 +91,7 @@ class PETSc(package.Package):
         if len(r)!=2: continue
         k = r[0].strip()
         v = r[1].strip()
+        v = v.replace('${PETSC_DIR}',self.dir)  # needed in some Cray installations
         if k == 'PETSC_SCALAR':
           self.scalar = v
         elif k == 'PETSC_PRECISION':
@@ -122,12 +128,17 @@ class PETSc(package.Package):
           self.sl_suffix = v
         elif k == 'RANLIB':
           self.ranlib = v
+        elif k == 'BLASLAPACK_LIB':
+          self.blaslapack_lib = v
+        elif k == 'SCALAPACK_LIB':
+          self.scalapack_lib = v
       f.close()
     except:
       self.log.Exit('Cannot process file ' + petscvariables)
 
     self.ind64 = False
     self.mpiuni = False
+    self.msmpi = False
     self.debug = False
     self.singlelib = False
     self.blaslapackmangling = ''
@@ -135,7 +146,7 @@ class PETSc(package.Package):
     self.fortran = False
     self.language = 'c'
     self.cxxdialectcxx11 = False
-    self.hpddm = False
+    self.packages = []
     try:
       f = open(petscconf_h)
       for l in f.readlines():
@@ -144,6 +155,8 @@ class PETSc(package.Package):
           self.ind64 = True
         elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MPIUNI' and l[2]=='1':
           self.mpiuni = True
+        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_MSMPI' and l[2]=='1':
+          self.msmpi = True
         elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_DEBUG' and l[2]=='1':
           self.debug = True
         elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_USE_SINGLE_LIBRARY' and l[2]=='1':
@@ -160,10 +173,12 @@ class PETSc(package.Package):
           self.language = 'c++'
         elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_CXX_DIALECT_CXX11' and l[2]=='1':
           self.cxxdialectcxx11 = True
-        elif len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_HPDDM' and l[2]=='1':
-          self.hpddm = True
         elif self.isinstall and len(l)==3 and l[0]=='#define' and l[1]=='PETSC_ARCH':
           self.arch = l[2].strip('"')
+        else:
+          for p in ['elemental','hpddm','mkl','mkl_pardiso','scalapack','slepc']:
+            if len(l)==3 and l[0]=='#define' and l[1]=='PETSC_HAVE_'+p.upper() and l[2]=='1':
+              self.packages.append(p)
       f.close()
     except:
       if self.isinstall:
@@ -171,3 +186,88 @@ class PETSc(package.Package):
       else:
         self.log.Exit('Cannot process file ' + petscconf_h)
 
+    if self.isinstall:
+      pseudoarch = 'arch-' + sys.platform.replace('cygwin','mswin')+ '-' + self.language.lower().replace('+','x')
+      if self.debug:
+        pseudoarch += '-debug'
+      else:
+        pseudoarch += '-opt'
+      if not 'real' in self.scalar:
+        pseudoarch += '-' + self.scalar
+      self.archname = 'installed-'+pseudoarch.replace('linux-','linux2-')
+    else:
+      self.archname = self.arch
+
+  def Process(self,slepcconf,slepcvars,slepcrules,slepc,petsc,archdir=''):
+    self.log.NewSection('Checking PETSc installation...')
+    if petsc.nversion > slepc.nversion:
+      self.log.Warn('PETSc version '+petsc.version+' is newer than SLEPc version '+slepc.version)
+    if slepc.release=='1' and not petsc.release=='1':
+      errmsg = 'A release version of SLEPc requires a release version of PETSc, not a development version'
+      if self.isrepo and slepc.isrepo:
+        errmsg += '\nType "git checkout release" (or "git checkout main") in both PETSc and SLEPc repositories'
+      self.log.Exit(errmsg)
+    if slepc.release=='0' and petsc.release=='1':
+      if not 'slepc' in petsc.packages:
+        errmsg = 'A development version of SLEPc cannot be built with a release version of PETSc'
+        if self.isrepo and slepc.isrepo:
+          errmsg += '\nType "git checkout release" (or "git checkout main") in both PETSc and SLEPc repositories'
+        self.log.Exit(errmsg)
+    if petsc.isinstall:
+      if os.path.realpath(petsc.prefixdir) != os.path.realpath(petsc.dir):
+        self.log.Warn('PETSC_DIR does not point to PETSc installation path')
+    petsc.Check()
+    if not petsc.havepackage:
+      self.log.Exit('Unable to link with PETSc')
+    if slepc.isrepo and not petsc.isinstall and petsc.branch!='release' and slepc.branch!='release':
+      try:
+        import dateutil.parser, datetime
+        petscdate = dateutil.parser.parse(petsc.gitdate)
+        slepcdate = dateutil.parser.parse(slepc.gitdate)
+        if abs(petscdate-slepcdate)>datetime.timedelta(days=30):
+          self.log.Warn('Your PETSc and SLEPc repos may not be in sync (more than 30 days apart)')
+      except ImportError: pass
+
+  def ShowInfo(self):
+    self.log.Println('PETSc directory:\n  '+self.dir)
+    if self.isrepo:
+      self.log.Println('  It is a git repository on branch: '+self.branch)
+    if self.isinstall:
+      self.log.Println('Prefix install with '+self.precision+' precision '+self.scalar+' numbers')
+    else:
+      self.log.Println('Architecture "'+self.archname+'" with '+self.precision+' precision '+self.scalar+' numbers')
+
+  def isGfortran100plus(self):
+    '''returns true if the compiler is gfortran-10.0.x or later'''
+    try:
+      (result, output) = self.RunCommand(self.fc+' --version',)
+      import re
+      strmatch = re.match('GNU Fortran\s+\(.*\)\s+(\d+)\.(\d+)',output)
+      if strmatch:
+        VMAJOR,VMINOR = strmatch.groups()
+        if (int(VMAJOR),int(VMINOR)) >= (10,0):
+          return 1
+    except RuntimeError:
+      pass
+
+  def removeWarningFlags(self,flags):
+    outflags = []
+    for flag in flags:
+      if not flag in ['-Werror','-Wall','-Wwrite-strings','-Wno-strict-aliasing','-Wno-unknown-pragmas','-Wno-unused-variable','-Wno-unused-dummy-argument','-fvisibility=hidden','-std=c89','-pedantic','--coverage','-Mfree','-fdefault-integer-8']:
+        outflags.append(flag)
+    return outflags
+
+  def getCFlags(self):
+    outflags = self.removeWarningFlags(self.cc_flags.split())
+    return ' '.join(outflags)
+
+  def getCXXFlags(self):
+    outflags = self.removeWarningFlags(self.cxx_flags.split())
+    return ' '.join(outflags)
+
+  def getFFlags(self):
+    outflags = self.removeWarningFlags(self.fc_flags.split())
+    if self.isGfortran100plus():
+      outflags.append('-fallow-argument-mismatch')
+    return ' '.join(outflags)
+
