@@ -1,7 +1,7 @@
 #
 #  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 #  SLEPc - Scalable Library for Eigenvalue Problem Computations
-#  Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+#  Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 #
 #  This file is part of SLEPc.
 #  SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -18,14 +18,14 @@ class Trlan(package.Package):
     self.installable    = True
     self.downloadable   = True
     self.version        = '201009'
-    self.url            = 'https://codeforge.lbl.gov/frs/download.php/210/trlan-'+self.version+'.tar.gz'
     self.archive        = 'trlan-'+self.version+'.tar.gz'
+    self.url            = 'https://codeforge.lbl.gov/frs/download.php/210/'+self.archive
     self.dirname        = 'trlan-'+self.version
     self.supportsscalar = ['real']
     self.fortran        = True
     self.ProcessArgs(argdb)
 
-  def Check(self,conf,vars,petsc,archdir):
+  def Check(self,slepcconf,slepcvars,petsc,archdir):
     functions = ['trlan77']
     if self.packagelibs:
       libs = [self.packagelibs]
@@ -36,32 +36,30 @@ class Trlan(package.Package):
         libs = [['-ltrlan_mpi']]
 
     if self.packagedir:
-      dirs = [os.path.join(self.packagedir,'lib'),self.packagedir]
+      dirs = [os.path.join(self.packagedir,'lib'),self.packagedir,os.path.join(self.packagedir,'lib64')]
     else:
       dirs = self.GenerateGuesses('TRLan',archdir)
 
-    self.FortranLib(conf,vars,dirs,libs,functions)
+    self.FortranLib(slepcconf,slepcvars,dirs,libs,functions)
 
 
-  def DownloadAndInstall(self,conf,vars,slepc,petsc,archdir,prefixdir):
+  def DownloadAndInstall(self,slepcconf,slepcvars,slepc,petsc,archdir,prefixdir):
     externdir = slepc.CreateDir(archdir,'externalpackages')
     builddir  = self.Download(externdir,slepc.downloaddir)
 
-    # Configure
-    g = open(os.path.join(builddir,'Make.inc'),'w')
-    g.write('FC     = '+petsc.fc+'\n')
-    g.write('F90    = '+petsc.fc+'\n')
-    g.write('FFLAGS = '+petsc.fc_flags.replace('-Wall','').replace('-Wshadow','')+'\n')
-    g.write('SHELL  = /bin/sh\n')
-    g.close()
+    # Makefile
+    cont  = 'FC     = '+petsc.fc+'\n'
+    cont += 'F90    = '+petsc.fc+'\n'
+    cont += 'FFLAGS = '+petsc.getFFlags()+'\n'
+    cont += 'SHELL  = /bin/sh\n'
+    self.WriteMakefile('Make.inc',builddir,cont)
 
     # Build package
     if petsc.mpiuni:
       target = 'lib'
     else:
       target = 'plib'
-    result,output = self.RunCommand('cd '+builddir+'&&'+petsc.make+' clean &&'+petsc.make+' '+target)
-    self.log.write(output)
+    (result,output) = self.RunCommand('cd '+builddir+'&&'+petsc.make+' clean &&'+petsc.make+' '+target)
     if result:
       self.log.Exit('Installation of TRLAN failed')
 
@@ -80,5 +78,5 @@ class Trlan(package.Package):
     else:
       libs = [['-ltrlan_mpi']]
     dirs = [libdir]
-    self.FortranLib(conf,vars,dirs,libs,functions)
+    self.FortranLib(slepcconf,slepcvars,dirs,libs,functions)
 
