@@ -3,18 +3,18 @@
 
 #define SLEPC_VERSION_RELEASE    1
 #define SLEPC_VERSION_MAJOR      3
-#define SLEPC_VERSION_MINOR      14
+#define SLEPC_VERSION_MINOR      15
 #define SLEPC_VERSION_SUBMINOR   0
 #define SLEPC_VERSION_PATCH      0
-#define SLEPC_RELEASE_DATE       "September 30, 2020"
-#define SLEPC_VERSION_DATE       "Sep 30, 2020"
+#define SLEPC_RELEASE_DATE       "March 31, 2021"
+#define SLEPC_VERSION_DATE       "Mar 31, 2021"
 
 #if !defined (SLEPC_VERSION_GIT)
-#define SLEPC_VERSION_GIT        "v3.14"
+#define SLEPC_VERSION_GIT        "v3.15"
 #endif
 
 #if !defined(SLEPC_VERSION_DATE_GIT)
-#define SLEPC_VERSION_DATE_GIT   "2020-09-30 17:08:23 +0200"
+#define SLEPC_VERSION_DATE_GIT   "2021-03-31 13:06:08 +0200"
 #endif
 
 #define SLEPC_VERSION_EQ(MAJOR,MINOR,SUBMINOR) \
