@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -33,6 +33,8 @@
 #include <slepc/private/epsimpl.h>                /*I "slepceps.h" I*/
 #include <slepcblaslapack.h>
 
+PetscLogEvent EPS_CISS_SVD;
+
 typedef struct {
   /* parameters */
   PetscInt          N;          /* number of integration points (32) */
@@ -356,56 +358,64 @@ static PetscErrorCode SolveLinearSystem(EPS eps,Mat A,Mat B,BV V,PetscInt L_star
 {
   PetscErrorCode ierr;
   EPS_CISS       *ctx = (EPS_CISS*)eps->data;
-  PetscInt       i,j,p_id;
-  Mat            Fz,kspMat;
-  Vec            Bvj,vj,yj;
+  PetscInt       i,p_id;
+  Mat            Fz,kspMat,MV,BMV=NULL,MC;
   KSP            ksp;
+  const char     *prefix;
 
   PetscFunctionBegin;
   if (!ctx->ksp) { ierr = EPSCISSGetKSPs(eps,&ctx->num_solve_point,&ctx->ksp);CHKERRQ(ierr); }
-  ierr = BVCreateVec(V,&Bvj);CHKERRQ(ierr);
   if (ctx->usest) {
     ierr = MatDuplicate(A,MAT_DO_NOT_COPY_VALUES,&Fz);CHKERRQ(ierr);
   }
+  ierr = BVSetActiveColumns(V,L_start,L_end);CHKERRQ(ierr);
+  ierr = BVGetMat(V,&MV);CHKERRQ(ierr);
+  if (B) {
+    ierr = MatProductCreate(B,MV,NULL,&BMV);CHKERRQ(ierr);
+    ierr = MatProductSetType(BMV,MATPRODUCT_AB);CHKERRQ(ierr);
+    ierr = MatProductSetFromOptions(BMV);CHKERRQ(ierr);
+    ierr = MatProductSymbolic(BMV);CHKERRQ(ierr);
+  }
   for (i=0;i<ctx->num_solve_point;i++) {
     p_id = i*ctx->subcomm->n + ctx->subcomm_id;
     if (!ctx->usest && initksp) {
       ierr = MatDuplicate(A,MAT_COPY_VALUES,&kspMat);CHKERRQ(ierr);
       if (B) {
-        ierr = MatAXPY(kspMat,-ctx->omega[p_id],B,DIFFERENT_NONZERO_PATTERN);CHKERRQ(ierr);
+        ierr = MatAXPY(kspMat,-ctx->omega[p_id],B,UNKNOWN_NONZERO_PATTERN);CHKERRQ(ierr);
       } else {
         ierr = MatShift(kspMat,-ctx->omega[p_id]);CHKERRQ(ierr);
       }
       ierr = KSPSetOperators(ctx->ksp[i],kspMat,kspMat);CHKERRQ(ierr);
+      /* set Mat prefix to be the same as KSP to enable setting command-line options (e.g. MUMPS) */
+      ierr = KSPGetOptionsPrefix(ctx->ksp[i],&prefix);CHKERRQ(ierr);
+      ierr = MatSetOptionsPrefix(kspMat,prefix);CHKERRQ(ierr);
       ierr = MatDestroy(&kspMat);CHKERRQ(ierr);
     } else if (ctx->usest) {
       ierr = STSetShift(eps->st,ctx->omega[p_id]);CHKERRQ(ierr);
       ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
     }
-    for (j=L_start;j<L_end;j++) {
-      ierr = BVGetColumn(V,j,&vj);CHKERRQ(ierr);
-      ierr = BVGetColumn(ctx->Y,i*ctx->L_max+j,&yj);CHKERRQ(ierr);
-      if (B) {
-        ierr = MatMult(B,vj,Bvj);CHKERRQ(ierr);
-        if (ctx->usest) {
-          ierr = KSPSolve(ksp,Bvj,yj);CHKERRQ(ierr);
-        } else {
-          ierr = KSPSolve(ctx->ksp[i],Bvj,yj);CHKERRQ(ierr);
-        }
+    ierr = BVSetActiveColumns(ctx->Y,i*ctx->L_max+L_start,i*ctx->L_max+L_end);CHKERRQ(ierr);
+    ierr = BVGetMat(ctx->Y,&MC);CHKERRQ(ierr);
+    if (B) {
+      ierr = MatProductNumeric(BMV);CHKERRQ(ierr);
+      if (ctx->usest) {
+        ierr = KSPMatSolve(ksp,BMV,MC);CHKERRQ(ierr);
       } else {
-        if (ctx->usest) {
-          ierr = KSPSolve(ksp,vj,yj);CHKERRQ(ierr);
-        } else {
-          ierr = KSPSolve(ctx->ksp[i],vj,yj);CHKERRQ(ierr);
-        }
+        ierr = KSPMatSolve(ctx->ksp[i],BMV,MC);CHKERRQ(ierr);
+      }
+    } else {
+      if (ctx->usest) {
+        ierr = KSPMatSolve(ksp,MV,MC);CHKERRQ(ierr);
+      } else {
+        ierr = KSPMatSolve(ctx->ksp[i],MV,MC);CHKERRQ(ierr);
       }
-      ierr = BVRestoreColumn(V,j,&vj);CHKERRQ(ierr);
-      ierr = BVRestoreColumn(ctx->Y,i*ctx->L_max+j,&yj);CHKERRQ(ierr);
     }
     if (ctx->usest && i<ctx->num_solve_point-1) { ierr = KSPReset(ksp);CHKERRQ(ierr); }
+    ierr = BVRestoreMat(ctx->Y,&MC);CHKERRQ(ierr);
   }
+  ierr = MatDestroy(&BMV);CHKERRQ(ierr);
+  ierr = BVRestoreMat(V,&MV);CHKERRQ(ierr);
   if (ctx->usest) { ierr = MatDestroy(&Fz);CHKERRQ(ierr); }
-  ierr = VecDestroy(&Bvj);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -459,15 +469,16 @@ static PetscErrorCode EstimateNumberEigs(EPS eps,PetscInt *L_add)
 
 static PetscErrorCode CalcMu(EPS eps,PetscScalar *Mu)
 {
-  PetscErrorCode ierr;
-  PetscMPIInt    sub_size,len;
-  PetscInt       i,j,k,s;
-  PetscScalar    *m,*temp,*temp2,*ppk,alp;
-  EPS_CISS       *ctx = (EPS_CISS*)eps->data;
-  Mat            M;
+  PetscErrorCode    ierr;
+  PetscMPIInt       sub_size,len;
+  PetscInt          i,j,k,s;
+  PetscScalar       *temp,*temp2,*ppk,alp;
+  const PetscScalar *pM;
+  EPS_CISS          *ctx = (EPS_CISS*)eps->data;
+  Mat               M;
 
   PetscFunctionBegin;
-  ierr = MPI_Comm_size(PetscSubcommChild(ctx->subcomm),&sub_size);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(PetscSubcommChild(ctx->subcomm),&sub_size);CHKERRMPI(ierr);
   ierr = PetscMalloc3(ctx->num_solve_point*ctx->L*(ctx->L+1),&temp,2*ctx->M*ctx->L*ctx->L,&temp2,ctx->num_solve_point,&ppk);CHKERRQ(ierr);
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,ctx->L,ctx->L_max*ctx->num_solve_point,NULL,&M);CHKERRQ(ierr);
   for (i=0;i<2*ctx->M*ctx->L*ctx->L;i++) temp2[i] = 0;
@@ -479,15 +490,15 @@ static PetscErrorCode CalcMu(EPS eps,PetscScalar *Mu)
     ierr = BVSetActiveColumns(ctx->V,0,ctx->L);CHKERRQ(ierr);
     ierr = BVDot(ctx->Y,ctx->V,M);CHKERRQ(ierr);
   }
-  ierr = MatDenseGetArray(M,&m);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(M,&pM);CHKERRQ(ierr);
   for (i=0;i<ctx->num_solve_point;i++) {
     for (j=0;j<ctx->L;j++) {
       for (k=0;k<ctx->L;k++) {
-        temp[k+j*ctx->L+i*ctx->L*ctx->L]=m[k+j*ctx->L+i*ctx->L*ctx->L_max];
+        temp[k+j*ctx->L+i*ctx->L*ctx->L]=pM[k+j*ctx->L+i*ctx->L*ctx->L_max];
       }
     }
   }
-  ierr = MatDenseRestoreArray(M,&m);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(M,&pM);CHKERRQ(ierr);
   for (i=0;i<ctx->num_solve_point;i++) ppk[i] = 1;
   for (k=0;k<2*ctx->M;k++) {
     for (j=0;j<ctx->L;j++) {
@@ -504,7 +515,7 @@ static PetscErrorCode CalcMu(EPS eps,PetscScalar *Mu)
   }
   for (i=0;i<2*ctx->M*ctx->L*ctx->L;i++) temp2[i] /= sub_size;
   ierr = PetscMPIIntCast(2*ctx->M*ctx->L*ctx->L,&len);CHKERRQ(ierr);
-  ierr = MPI_Allreduce(temp2,Mu,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)eps));CHKERRQ(ierr);
+  ierr = MPIU_Allreduce(temp2,Mu,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)eps));CHKERRQ(ierr);
   ierr = PetscFree3(temp,temp2,ppk);CHKERRQ(ierr);
   ierr = MatDestroy(&M);CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -535,6 +546,7 @@ static PetscErrorCode SVD_H0(EPS eps,PetscScalar *S,PetscInt *K)
 #endif
 
   PetscFunctionBegin;
+  ierr = PetscLogEventBegin(EPS_CISS_SVD,eps,0,0,0);CHKERRQ(ierr);
   ierr = PetscMalloc1(5*ml,&work);CHKERRQ(ierr);
 #if defined(PETSC_USE_COMPLEX)
   ierr = PetscMalloc1(5*ml,&rwork);CHKERRQ(ierr);
@@ -557,6 +569,7 @@ static PetscErrorCode SVD_H0(EPS eps,PetscScalar *S,PetscInt *K)
 #if defined(PETSC_USE_COMPLEX)
   ierr = PetscFree(rwork);CHKERRQ(ierr);
 #endif
+  ierr = PetscLogEventEnd(EPS_CISS_SVD,eps,0,0,0);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -642,7 +655,7 @@ static PetscErrorCode SVD_S(BV S,PetscInt ml,PetscReal delta,PetscReal *sigma,Pe
     }
     ierr = PetscArrayzero(temp2,ml*ml);CHKERRQ(ierr);
     ierr = PetscMPIIntCast(ml*ml,&len);CHKERRQ(ierr);
-    ierr = MPI_Allreduce(temp,temp2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)S));CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(temp,temp2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)S));CHKERRQ(ierr);
 
     ierr = PetscBLASIntCast(ml,&m);CHKERRQ(ierr);
     n = m; lda = m; lwork = 5*m, ldu = 1; ldvt = 1;
@@ -818,12 +831,13 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_CISS       *ctx = (EPS_CISS*)eps->data;
-  PetscBool      issinvert,istrivial,isring,isellipse,isinterval,flg,useconj;
+  PetscBool      istrivial,isring,isellipse,isinterval,flg,useconj;
   PetscReal      c,d;
+  PetscRandom    rand;
   Mat            A;
 
   PetscFunctionBegin;
-  if (!eps->ncv) {
+  if (eps->ncv==PETSC_DEFAULT) {
     eps->ncv = ctx->L_max*ctx->M;
     if (eps->ncv>eps->n) {
       eps->ncv = eps->n;
@@ -839,13 +853,11 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
     }
   }
   ctx->L = PetscMin(ctx->L,ctx->L_max);
-  if (!eps->max_it) eps->max_it = 1;
-  if (!eps->mpd) eps->mpd = eps->ncv;
+  if (eps->max_it==PETSC_DEFAULT) eps->max_it = 1;
+  if (eps->mpd==PETSC_DEFAULT) eps->mpd = eps->ncv;
   if (!eps->which) eps->which = EPS_ALL;
-  if (!eps->extraction) { ierr = EPSSetExtraction(eps,EPS_RITZ);CHKERRQ(ierr); }
-  else if (eps->extraction!=EPS_RITZ) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Unsupported extraction type");
-  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
-  if (eps->stopping!=EPSStoppingBasic) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support user-defined stopping test");
+  if (eps->which!=EPS_ALL) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver supports only computing all eigenvalues");
+  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_EXTRACTION | EPS_FEATURE_STOPPING | EPS_FEATURE_TWOSIDED);
 
   /* check region */
   ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
@@ -861,9 +873,7 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
   if (useconj!=ctx->useconj) { ierr = EPSCISSResetSubcomm(eps);CHKERRQ(ierr); }
 
 #if !defined(PETSC_USE_COMPLEX)
-  if (isring) {
-    SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Ring region only supported for complex scalars");
-  }
+  if (isring) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Ring region only supported for complex scalars");
 #endif
   if (isinterval) {
     ierr = RGIntervalGetEndpoints(eps->rg,NULL,NULL,&c,&d);CHKERRQ(ierr);
@@ -878,6 +888,7 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
   if (!ctx->subcomm) { ierr = EPSCISSSetUpSubComm(eps,&ctx->num_solve_point);CHKERRQ(ierr); }
 
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
+  ierr = BVGetRandomContext(eps->V,&rand);CHKERRQ(ierr);  /* make sure the random context is available when duplicating */
   if (ctx->weight) { ierr = PetscFree4(ctx->weight,ctx->omega,ctx->pp,ctx->sigma);CHKERRQ(ierr); }
   ierr = PetscMalloc4(ctx->N,&ctx->weight,ctx->N+1,&ctx->omega,ctx->N,&ctx->pp,ctx->L_max*ctx->M,&ctx->sigma);CHKERRQ(ierr);
   ierr = PetscLogObjectMemory((PetscObject)eps,3*ctx->N*sizeof(PetscScalar)+ctx->L_max*ctx->N*sizeof(PetscReal));CHKERRQ(ierr);
@@ -891,7 +902,7 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
   ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->V);CHKERRQ(ierr);
 
   ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)A,MATSHELL,&flg);CHKERRQ(ierr);
+  ierr = MatIsShell(A,&flg);CHKERRQ(ierr);
   if (flg) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Matrix type shell is not supported in this solver");
 
   if (!ctx->usest_set) ctx->usest = (ctx->npart>1)? PETSC_FALSE: PETSC_TRUE;
@@ -908,10 +919,7 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
     ierr = PetscLogObjectParent((PetscObject)eps,(PetscObject)ctx->pV);CHKERRQ(ierr);
   }
 
-  if (ctx->usest) {
-    ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&issinvert);CHKERRQ(ierr);
-    if (!issinvert) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"If the usest flag is set, you must select the STSINVERT spectral transformation");
-  }
+  EPSCheckSinvertCondition(eps,ctx->usest," (with the usest flag set)");
 
   ierr = BVDestroy(&ctx->Y);CHKERRQ(ierr);
   if (ctx->pA) {
@@ -940,14 +948,37 @@ PetscErrorCode EPSSetUp_CISS(EPS eps)
     }
   }
   ierr = DSAllocate(eps->ds,eps->ncv);CHKERRQ(ierr);
-  ierr = EPSSetWorkVecs(eps,2);CHKERRQ(ierr);
 
 #if !defined(PETSC_USE_COMPLEX)
+  ierr = EPSSetWorkVecs(eps,3);CHKERRQ(ierr);
   if (!eps->ishermitian) { ierr = PetscInfo(eps,"Warning: complex eigenvalues are not calculated exactly without --with-scalar-type=complex in PETSc\n");CHKERRQ(ierr); }
+#else
+  ierr = EPSSetWorkVecs(eps,2);CHKERRQ(ierr);
 #endif
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode EPSSetUpSort_CISS(EPS eps)
+{
+  PetscErrorCode ierr;
+  SlepcSC        sc;
+
+  PetscFunctionBegin;
+  /* fill sorting criterion context */
+  eps->sc->comparison    = SlepcCompareSmallestReal;
+  eps->sc->comparisonctx = NULL;
+  eps->sc->map           = NULL;
+  eps->sc->mapobj        = NULL;
+
+  /* fill sorting criterion for DS */
+  ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
+  sc->comparison    = SlepcCompareLargestMagnitude;
+  sc->comparisonctx = NULL;
+  sc->map           = NULL;
+  sc->mapobj        = NULL;
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode EPSSolve_CISS(EPS eps)
 {
   PetscErrorCode ierr;
@@ -957,23 +988,22 @@ PetscErrorCode EPSSolve_CISS(EPS eps)
   PetscScalar    *Mu,*H0,*H1=NULL,*rr,*temp;
   PetscReal      error,max_error,norm;
   PetscBool      *fl1;
-  Vec            si,w[3];
-  SlepcSC        sc;
+  Vec            si,si1=NULL,w[3];
   PetscRandom    rand;
 #if defined(PETSC_USE_COMPLEX)
   PetscBool      isellipse;
+#else
+  PetscReal      normi;
 #endif
 
   PetscFunctionBegin;
   w[0] = eps->work[0];
+#if defined(PETSC_USE_COMPLEX)
   w[1] = NULL;
+#else
+  w[1] = eps->work[2];
+#endif
   w[2] = eps->work[1];
-  /* override SC settings */
-  ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
-  sc->comparison    = SlepcCompareLargestMagnitude;
-  sc->comparisonctx = NULL;
-  sc->map           = NULL;
-  sc->mapobj        = NULL;
   ierr = VecGetLocalSize(w[0],&nlocal);CHKERRQ(ierr);
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   ierr = STGetNumMatrices(eps->st,&nmat);CHKERRQ(ierr);
@@ -1045,7 +1075,9 @@ PetscErrorCode EPSSolve_CISS(EPS eps)
         ierr = ConstructS(eps);CHKERRQ(ierr);
         ierr = BVSetActiveColumns(ctx->S,0,ctx->L);CHKERRQ(ierr);
         ierr = BVCopy(ctx->S,ctx->V);CHKERRQ(ierr);
+        ierr = PetscLogEventBegin(EPS_CISS_SVD,eps,0,0,0);CHKERRQ(ierr);
         ierr = SVD_S(ctx->S,ctx->L*ctx->M,ctx->delta,ctx->sigma,&nv);CHKERRQ(ierr);
+        ierr = PetscLogEventEnd(EPS_CISS_SVD,eps,0,0,0);CHKERRQ(ierr);
         if (ctx->sigma[0]>ctx->delta && nv==ctx->L*ctx->M && inner!=ctx->refine_inner) {
           if (ctx->pA) {
             ierr = VecScatterVecs(eps,ctx->V,ctx->L);CHKERRQ(ierr);
@@ -1129,13 +1161,28 @@ PetscErrorCode EPSSolve_CISS(EPS eps)
       max_error = 0.0;
       for (i=0;i<eps->nconv;i++) {
         ierr = BVGetColumn(ctx->S,i,&si);CHKERRQ(ierr);
-        ierr = EPSComputeResidualNorm_Private(eps,PETSC_FALSE,eps->eigr[i],eps->eigi[i],si,NULL,w,&error);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+        if (eps->eigi[i]!=0.0) { ierr = BVGetColumn(ctx->S,i+1,&si1);CHKERRQ(ierr); }
+#endif
+        ierr = EPSComputeResidualNorm_Private(eps,PETSC_FALSE,eps->eigr[i],eps->eigi[i],si,si1,w,&error);CHKERRQ(ierr);
         if (ctx->extraction == EPS_CISS_EXTRACTION_HANKEL) {  /* vector is not normalized */
           ierr = VecNorm(si,NORM_2,&norm);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+          if (eps->eigi[i]!=0.0) {
+            ierr = VecNorm(si1,NORM_2,&normi);CHKERRQ(ierr);
+            norm = SlepcAbsEigenvalue(norm,normi);
+          }
+#endif
           error /= norm;
         }
         ierr = (*eps->converged)(eps,eps->eigr[i],eps->eigi[i],error,&error,eps->convergedctx);CHKERRQ(ierr);
         ierr = BVRestoreColumn(ctx->S,i,&si);CHKERRQ(ierr);
+#if !defined(PETSC_USE_COMPLEX)
+        if (eps->eigi[i]!=0.0) {
+          ierr = BVRestoreColumn(ctx->S,i+1,&si1);CHKERRQ(ierr);
+          i++;
+        }
+#endif
         max_error = PetscMax(max_error,error);
       }
 
@@ -1144,12 +1191,12 @@ PetscErrorCode EPSSolve_CISS(EPS eps)
       else {
         if (eps->nconv > ctx->L) {
           ierr = MatCreateSeqDense(PETSC_COMM_SELF,eps->nconv,ctx->L,NULL,&M);CHKERRQ(ierr);
-          ierr = MatDenseGetArray(M,&temp);CHKERRQ(ierr);
+          ierr = MatDenseGetArrayWrite(M,&temp);CHKERRQ(ierr);
           for (i=0;i<ctx->L*eps->nconv;i++) {
             ierr = PetscRandomGetValue(rand,&temp[i]);CHKERRQ(ierr);
             temp[i] = PetscRealPart(temp[i]);
           }
-          ierr = MatDenseRestoreArray(M,&temp);CHKERRQ(ierr);
+          ierr = MatDenseRestoreArrayWrite(M,&temp);CHKERRQ(ierr);
           ierr = BVSetActiveColumns(ctx->S,0,eps->nconv);CHKERRQ(ierr);
           ierr = BVMultInPlace(ctx->S,M,0,ctx->L);CHKERRQ(ierr);
           ierr = MatDestroy(&M);CHKERRQ(ierr);
@@ -1176,21 +1223,37 @@ PetscErrorCode EPSComputeVectors_CISS(EPS eps)
 {
   PetscErrorCode ierr;
   EPS_CISS       *ctx = (EPS_CISS*)eps->data;
-  PetscReal      norm;
-  PetscInt       i;
-  Mat            B=NULL;
+  PetscInt       n;
+  Mat            Z,B=NULL;
 
   PetscFunctionBegin;
-  ierr = EPSComputeVectors_Schur(eps);CHKERRQ(ierr);
-  if (ctx->extraction == EPS_CISS_EXTRACTION_HANKEL) { /* normalize */
+  if (eps->ishermitian) {
+    if (eps->isgeneralized && !eps->ispositive) {
+      ierr = EPSComputeVectors_Indefinite(eps);CHKERRQ(ierr);
+    } else {
+      ierr = EPSComputeVectors_Hermitian(eps);CHKERRQ(ierr);
+    }
+    PetscFunctionReturn(0);
+  }
+  ierr = DSGetDimensions(eps->ds,&n,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
+  ierr = BVSetActiveColumns(eps->V,0,n);CHKERRQ(ierr);
+
+  /* right eigenvectors */
+  ierr = DSVectors(eps->ds,DS_MAT_X,NULL,NULL);CHKERRQ(ierr);
+
+  /* V = V * Z */
+  ierr = DSGetMat(eps->ds,DS_MAT_X,&Z);CHKERRQ(ierr);
+  ierr = BVMultInPlace(eps->V,Z,0,n);CHKERRQ(ierr);
+  ierr = MatDestroy(&Z);CHKERRQ(ierr);
+  ierr = BVSetActiveColumns(eps->V,0,eps->nconv);CHKERRQ(ierr);
+
+  /* normalize */
+  if (ctx->extraction == EPS_CISS_EXTRACTION_HANKEL) {
     if (eps->isgeneralized && eps->ishermitian && eps->ispositive) {
       ierr = STGetMatrix(eps->st,1,&B);CHKERRQ(ierr);
       ierr = BVSetMatrix(eps->V,B,PETSC_FALSE);CHKERRQ(ierr);
     }
-    for (i=0;i<eps->nconv;i++) {
-      ierr = BVNormColumn(eps->V,i,NORM_2,&norm);CHKERRQ(ierr);
-      ierr = BVScaleColumn(eps->V,i,1.0/norm);CHKERRQ(ierr);
-    }
+    ierr = BVNormalize(eps->V,NULL);CHKERRQ(ierr);
     if (B) { ierr = BVSetMatrix(eps->V,NULL,PETSC_FALSE);CHKERRQ(ierr); }
   }
   PetscFunctionReturn(0);
@@ -1724,7 +1787,7 @@ static PetscErrorCode EPSCISSGetExtraction_CISS(EPS eps,EPSCISSExtraction *extra
 .  eps - the eigenproblem solver context
 
    Output Parameters:
-+  extraction - extraction technique
+.  extraction - extraction technique
 
    Level: advanced
 
@@ -1973,6 +2036,7 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_CISS(EPS eps)
 
   eps->ops->solve          = EPSSolve_CISS;
   eps->ops->setup          = EPSSetUp_CISS;
+  eps->ops->setupsort      = EPSSetUpSort_CISS;
   eps->ops->setfromoptions = EPSSetFromOptions_CISS;
   eps->ops->destroy        = EPSDestroy_CISS;
   eps->ops->reset          = EPSReset_CISS;
@@ -1993,6 +2057,10 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_CISS(EPS eps)
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSCISSSetExtraction_C",EPSCISSSetExtraction_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSCISSGetExtraction_C",EPSCISSGetExtraction_CISS);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSCISSGetKSPs_C",EPSCISSGetKSPs_CISS);CHKERRQ(ierr);
+
+  /* log events */
+  ierr = PetscLogEventRegister("EPSCISS_SVD",EPS_CLASSID,&EPS_CISS_SVD);CHKERRQ(ierr);
+
   /* set default values of parameters */
   ctx->N                  = 32;
   ctx->L                  = 16;
