@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -16,55 +16,7 @@
 #include <slepcblaslapack.h>
 
 /*
-   EPSBasicArnoldi - Computes an m-step Arnoldi factorization. The first k
-   columns are assumed to be locked and therefore they are not modified. On
-   exit, the following relation is satisfied:
-
-                    OP * V - V * H = beta*v_m * e_m^T
-
-   where the columns of V are the Arnoldi vectors (which are B-orthonormal),
-   H is an upper Hessenberg matrix, e_m is the m-th vector of the canonical basis.
-   On exit, beta contains the B-norm of V[m] before normalization.
-*/
-PetscErrorCode EPSBasicArnoldi(EPS eps,PetscBool trans,PetscScalar *H,PetscInt ldh,PetscInt k,PetscInt *M,PetscReal *beta,PetscBool *breakdown)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *a;
-  PetscInt       j,nc,n,m = *M;
-  Vec            vj,vj1,buf;
-
-  PetscFunctionBegin;
-  ierr = BVSetActiveColumns(eps->V,0,m);CHKERRQ(ierr);
-  for (j=k;j<m;j++) {
-    ierr = BVGetColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVGetColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    if (trans) {
-      ierr = STApplyTranspose(eps->st,vj,vj1);CHKERRQ(ierr);
-    } else {
-      ierr = STApply(eps->st,vj,vj1);CHKERRQ(ierr);
-    }
-    ierr = BVRestoreColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = BVOrthonormalizeColumn(eps->V,j+1,PETSC_FALSE,beta,breakdown);CHKERRQ(ierr);
-    if (*breakdown) {
-      *M = j+1;
-      break;
-    }
-  }
-  /* extract Hessenberg matrix from the BV object */
-  ierr = BVGetNumConstraints(eps->V,&nc);CHKERRQ(ierr);
-  ierr = BVGetSizes(eps->V,NULL,NULL,&n);CHKERRQ(ierr);
-  ierr = BVGetBufferVec(eps->V,&buf);CHKERRQ(ierr);
-  ierr = VecGetArray(buf,&a);CHKERRQ(ierr);
-  for (j=k;j<*M;j++) {
-    ierr = PetscMemcpy(H+j*ldh,a+nc+(j+1)*(nc+n),(j+2)*sizeof(PetscScalar));CHKERRQ(ierr);
-  }
-  ierr = VecRestoreArray(buf,&a);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
-/*
-   EPSDelayedArnoldi - This function is equivalent to EPSBasicArnoldi but
+   EPSDelayedArnoldi - This function is equivalent to BVMatArnoldi but
    performs the computation in a different way. The main idea is that
    reorthogonalization is delayed to the next Arnoldi step. This version is
    more scalable but in some cases convergence may stagnate.
@@ -273,14 +225,14 @@ PetscErrorCode EPSKrylovConvergence_Filter(EPS eps,PetscBool getall,PetscInt kin
    Output Parameters:
      kout  - the first index where the convergence test failed
 */
-PetscErrorCode EPSKrylovConvergence(EPS eps,PetscBool getall,PetscInt kini,PetscInt nits,PetscReal beta,PetscReal corrf,PetscInt *kout)
+PetscErrorCode EPSKrylovConvergence(EPS eps,PetscBool getall,PetscInt kini,PetscInt nits,PetscReal beta,PetscReal betat,PetscReal corrf,PetscInt *kout)
 {
   PetscErrorCode ierr;
-  PetscInt       k,newk,marker,ld,inside;
+  PetscInt       k,newk,newk2,marker,ld,inside;
   PetscScalar    re,im,*Zr,*Zi,*X;
-  PetscReal      resnorm,gamma;
+  PetscReal      resnorm,gamma,lerrest;
   PetscBool      isshift,isfilter,refined,istrivial;
-  Vec            x,y,w[3];
+  Vec            x=NULL,y=NULL,w[3];
 
   PetscFunctionBegin;
   if (eps->which == EPS_ALL) {
@@ -332,12 +284,20 @@ PetscErrorCode EPSKrylovConvergence(EPS eps,PetscBool getall,PetscInt kini,Petsc
       else Zi = NULL;
       ierr = EPSComputeRitzVector(eps,Zr,Zi,eps->V,x,y);CHKERRQ(ierr);
       ierr = DSRestoreArray(eps->ds,DS_MAT_X,&X);CHKERRQ(ierr);
-      ierr = EPSComputeResidualNorm_Private(eps,re,im,x,y,w,&resnorm);CHKERRQ(ierr);
+      ierr = EPSComputeResidualNorm_Private(eps,PETSC_FALSE,re,im,x,y,w,&resnorm);CHKERRQ(ierr);
     }
     else if (!refined) resnorm *= beta*corrf;
     /* error estimate */
     ierr = (*eps->converged)(eps,re,im,resnorm,&eps->errest[k],eps->convergedctx);CHKERRQ(ierr);
     if (marker==-1 && eps->errest[k] >= eps->tol) marker = k;
+    if (eps->twosided) {
+      newk2 = k;
+      ierr = DSVectors(eps->ds,DS_MAT_Y,&newk2,&resnorm);CHKERRQ(ierr);
+      resnorm *= betat;
+      ierr = (*eps->converged)(eps,re,im,resnorm,&lerrest,eps->convergedctx);CHKERRQ(ierr);
+      eps->errest[k] = PetscMax(eps->errest[k],lerrest);
+      if (marker==-1 && lerrest >= eps->tol) marker = k;
+    }
     if (newk==k+1) {
       eps->errest[k+1] = eps->errest[k];
       k++;
@@ -358,55 +318,6 @@ PetscErrorCode EPSKrylovConvergence(EPS eps,PetscBool getall,PetscInt kini,Petsc
   PetscFunctionReturn(0);
 }
 
-/*
-   EPSFullLanczos - Computes an m-step Lanczos factorization with full
-   reorthogonalization.  At each Lanczos step, the corresponding Lanczos
-   vector is orthogonalized with respect to all previous Lanczos vectors.
-   This is equivalent to computing an m-step Arnoldi factorization and
-   exploting symmetry of the operator.
-
-   The first k columns are assumed to be locked and therefore they are
-   not modified. On exit, the following relation is satisfied:
-
-                    OP * V - V * T = beta_m*v_m * e_m^T
-
-   where the columns of V are the Lanczos vectors (which are B-orthonormal),
-   T is a real symmetric tridiagonal matrix, and e_m is the m-th vector of
-   the canonical basis. The tridiagonal is stored as two arrays: alpha
-   contains the diagonal elements, beta the off-diagonal. On exit, the last
-   element of beta contains the B-norm of V[m] before normalization.
-*/
-PetscErrorCode EPSFullLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,PetscInt k,PetscInt *M,PetscBool *breakdown)
-{
-  PetscErrorCode ierr;
-  PetscScalar    *a;
-  PetscInt       j,nc,n,m = *M;
-  Vec            vj,vj1,buf;
-
-  PetscFunctionBegin;
-  ierr = BVSetActiveColumns(eps->V,0,m);CHKERRQ(ierr);
-  for (j=k;j<m;j++) {
-    ierr = BVGetColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVGetColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = STApply(eps->st,vj,vj1);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j,&vj);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(eps->V,j+1,&vj1);CHKERRQ(ierr);
-    ierr = BVOrthonormalizeColumn(eps->V,j+1,PETSC_FALSE,beta+j,breakdown);CHKERRQ(ierr);
-    if (*breakdown) {
-      *M = j+1;
-      break;
-    }
-  }
-  /* extract tridiagonal matrix from the BV object (only alpha, beta is already in its place) */
-  ierr = BVGetNumConstraints(eps->V,&nc);CHKERRQ(ierr);
-  ierr = BVGetSizes(eps->V,NULL,NULL,&n);CHKERRQ(ierr);
-  ierr = BVGetBufferVec(eps->V,&buf);CHKERRQ(ierr);
-  ierr = VecGetArray(buf,&a);CHKERRQ(ierr);
-  for (j=k;j<*M;j++) alpha[j] = PetscRealPart(a[nc+j+(j+1)*(nc+n)]);
-  ierr = VecRestoreArray(buf,&a);CHKERRQ(ierr);
-  PetscFunctionReturn(0);
-}
-
 PetscErrorCode EPSPseudoLanczos(EPS eps,PetscReal *alpha,PetscReal *beta,PetscReal *omega,PetscInt k,PetscInt *M,PetscBool *breakdown,PetscBool *symmlost,PetscReal *cos,Vec w)
 {
   PetscErrorCode ierr;
