@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -93,9 +93,11 @@ static PetscErrorCode EstimateRange(Mat A,PetscReal *left,PetscReal *right)
 
 static PetscErrorCode EPSSetUp_KrylovSchur_Filter(EPS eps)
 {
-  PetscErrorCode ierr;
-  PetscReal      rleft,rright;
-  Mat            A;
+  PetscErrorCode  ierr;
+  EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
+  PetscBool       estimaterange=PETSC_TRUE;
+  PetscReal       rleft,rright;
+  Mat             A;
 
   PetscFunctionBegin;
   EPSCheckHermitianCondition(eps,PETSC_TRUE," with polynomial filter");
@@ -104,12 +106,16 @@ static PetscErrorCode EPSSetUp_KrylovSchur_Filter(EPS eps)
   EPSCheckUnsupportedCondition(eps,EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_EXTRACTION,PETSC_TRUE," with polynomial filter");
   if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL*1e-2;  /* use tighter tolerance */
   ierr = STFilterSetInterval(eps->st,eps->inta,eps->intb);CHKERRQ(ierr);
-  ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
-  ierr = STFilterGetRange(eps->st,&rleft,&rright);CHKERRQ(ierr);
-  if (!rleft && !rright) {
+  if (!ctx->estimatedrange) {
+    ierr = STFilterGetRange(eps->st,&rleft,&rright);CHKERRQ(ierr);
+    estimaterange = (!rleft && !rright)? PETSC_TRUE: PETSC_FALSE;
+  }
+  if (estimaterange) { /* user did not set a range */
+    ierr = STGetMatrix(eps->st,0,&A);CHKERRQ(ierr);
     ierr = EstimateRange(A,&rleft,&rright);CHKERRQ(ierr);
     ierr = PetscInfo2(eps,"Setting eigenvalue range to [%g,%g]\n",(double)rleft,(double)rright);CHKERRQ(ierr);
     ierr = STFilterSetRange(eps->st,rleft,rright);CHKERRQ(ierr);
+    ctx->estimatedrange = PETSC_TRUE;
   }
   if (eps->ncv==PETSC_DEFAULT && eps->nev==1) eps->nev = 40;  /* user did not provide nev estimation */
   ierr = EPSSetDimensions_Default(eps,eps->nev,&eps->ncv,&eps->mpd);CHKERRQ(ierr);
@@ -186,11 +192,12 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
       eps->ops->solve = EPSSolve_KrylovSchur_Default;
       eps->ops->computevectors = EPSComputeVectors_Schur;
       ierr = DSSetType(eps->ds,DSNHEP);CHKERRQ(ierr);
+      ierr = DSSetExtraRow(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
       ierr = DSAllocate(eps->ds,eps->ncv+1);CHKERRQ(ierr);
       break;
     case EPS_KS_SYMM:
     case EPS_KS_FILTER:
-      eps->ops->solve = EPSSolve_KrylovSchur_Symm;
+      eps->ops->solve = EPSSolve_KrylovSchur_Default;
       eps->ops->computevectors = EPSComputeVectors_Hermitian;
       ierr = DSSetType(eps->ds,DSHEP);CHKERRQ(ierr);
       ierr = DSSetCompact(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
@@ -206,6 +213,7 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
       eps->ops->computevectors = EPSComputeVectors_Indefinite;
       ierr = DSSetType(eps->ds,DSGHIEP);CHKERRQ(ierr);
       ierr = DSSetCompact(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
+      ierr = DSSetExtraRow(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
       ierr = DSAllocate(eps->ds,eps->ncv+1);CHKERRQ(ierr);
       /* force reorthogonalization for pseudo-Lanczos */
       ierr = BVGetOrthogonalization(eps->V,&otype,NULL,&eta,&obtype);CHKERRQ(ierr);
@@ -214,10 +222,9 @@ PetscErrorCode EPSSetUp_KrylovSchur(EPS eps)
     case EPS_KS_TWOSIDED:
       eps->ops->solve = EPSSolve_KrylovSchur_TwoSided;
       eps->ops->computevectors = EPSComputeVectors_Schur;
-      ierr = DSSetType(eps->ds,DSNHEP);CHKERRQ(ierr);
+      ierr = DSSetType(eps->ds,DSNHEPTS);CHKERRQ(ierr);
       ierr = DSAllocate(eps->ds,eps->ncv+1);CHKERRQ(ierr);
-      ierr = DSSetType(eps->dsts,DSNHEP);CHKERRQ(ierr);
-      ierr = DSAllocate(eps->dsts,eps->ncv+1);CHKERRQ(ierr);
+      ierr = DSSetExtraRow(eps->ds,PETSC_TRUE);CHKERRQ(ierr);
       break;
     default: SETERRQ(PetscObjectComm((PetscObject)eps),1,"Unexpected error");
   }
@@ -260,15 +267,16 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
-  PetscInt        i,j,*pj,k,l,nv,ld,nconv;
+  PetscInt        j,*pj,k,l,nv,ld,nconv;
   Mat             U,Op;
-  PetscScalar     *S,*Q,*g;
-  PetscReal       beta,gamma=1.0;
-  PetscBool       breakdown,harmonic;
+  PetscScalar     *S,*g;
+  PetscReal       beta,gamma=1.0,*a,*b;
+  PetscBool       breakdown,harmonic,hermitian;
 
   PetscFunctionBegin;
   ierr = DSGetLeadingDimension(eps->ds,&ld);CHKERRQ(ierr);
   harmonic = (eps->extraction==EPS_HARMONIC || eps->extraction==EPS_REFINED_HARMONIC)?PETSC_TRUE:PETSC_FALSE;
+  hermitian = (eps->ishermitian && !harmonic)?PETSC_TRUE:PETSC_FALSE;
   if (harmonic) { ierr = PetscMalloc1(ld,&g);CHKERRQ(ierr); }
   if (eps->arbitrary) pj = &j;
   else pj = NULL;
@@ -284,9 +292,17 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
     /* Compute an nv-step Arnoldi factorization */
     nv = PetscMin(eps->nconv+eps->mpd,eps->ncv);
     ierr = STGetOperator(eps->st,&Op);CHKERRQ(ierr);
-    ierr = DSGetArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-    ierr = BVMatArnoldi(eps->V,Op,S,ld,eps->nconv+l,&nv,&beta,&breakdown);CHKERRQ(ierr);
-    ierr = DSRestoreArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
+    if (hermitian) {
+      ierr = DSGetArrayReal(eps->ds,DS_MAT_T,&a);CHKERRQ(ierr);
+      b = a + ld;
+      ierr = BVMatLanczos(eps->V,Op,a,b,eps->nconv+l,&nv,&breakdown);CHKERRQ(ierr);
+      beta = b[nv-1];
+      ierr = DSRestoreArrayReal(eps->ds,DS_MAT_T,&a);CHKERRQ(ierr);
+    } else {
+      ierr = DSGetArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
+      ierr = BVMatArnoldi(eps->V,Op,S,ld,eps->nconv+l,&nv,&beta,&breakdown);CHKERRQ(ierr);
+      ierr = DSRestoreArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
+    }
     ierr = STRestoreOperator(eps->st,&Op);CHKERRQ(ierr);
     ierr = DSSetDimensions(eps->ds,nv,0,eps->nconv,eps->nconv+l);CHKERRQ(ierr);
     if (l==0) {
@@ -308,6 +324,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
       j=1;
     }
     ierr = DSSort(eps->ds,eps->eigr,eps->eigi,eps->rr,eps->ri,pj);CHKERRQ(ierr);
+    ierr = DSUpdateExtraRow(eps->ds);CHKERRQ(ierr);
     ierr = DSSynchronize(eps->ds,eps->eigr,eps->eigi);CHKERRQ(ierr);
 
     /* Check convergence */
@@ -319,14 +336,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
     if (eps->reason != EPS_CONVERGED_ITERATING || breakdown || k==nv) l = 0;
     else {
       l = PetscMax(1,(PetscInt)((nv-k)*ctx->keep));
-#if !defined(PETSC_USE_COMPLEX)
-      ierr = DSGetArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-      if (S[k+l+(k+l-1)*ld] != 0.0) {
-        if (k+l<nv-1) l = l+1;
-        else l = l-1;
-      }
-      ierr = DSRestoreArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-#endif
+      if (!hermitian) { ierr = DSGetTruncateSize(eps->ds,k,nv,&l);CHKERRQ(ierr); }
     }
     if (!ctx->lock && l>0) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
     if (l) { ierr = PetscInfo1(eps,"Preparing to restart keeping l=%D vectors\n",l);CHKERRQ(ierr); }
@@ -352,15 +362,10 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
           ierr = BVMultColumn(eps->V,-1.0,1.0,nv,g);CHKERRQ(ierr);
           ierr = BVScaleColumn(eps->V,nv,1.0/gamma);CHKERRQ(ierr);
           ierr = BVSetActiveColumns(eps->V,eps->nconv,nv);CHKERRQ(ierr);
+          ierr = DSSetDimensions(eps->ds,nv,0,k,nv);CHKERRQ(ierr);
         }
         /* Prepare the Rayleigh quotient for restart */
-        ierr = DSGetArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-        ierr = DSGetArray(eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-        for (i=k;i<k+l;i++) {
-          S[k+l+i*ld] = Q[nv-1+i*ld]*beta*gamma;
-        }
-        ierr = DSRestoreArray(eps->ds,DS_MAT_A,&S);CHKERRQ(ierr);
-        ierr = DSRestoreArray(eps->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
+        ierr = DSTruncate(eps->ds,k+l,PETSC_FALSE);CHKERRQ(ierr);
       }
     }
     /* Update the corresponding vectors V(:,idx) = V*Q(:,idx) */
@@ -376,10 +381,7 @@ PetscErrorCode EPSSolve_KrylovSchur_Default(EPS eps)
   }
 
   if (harmonic) { ierr = PetscFree(g);CHKERRQ(ierr); }
-  /* truncate Schur decomposition and change the state to raw so that
-     DSVectors() computes eigenvectors from scratch */
-  ierr = DSSetDimensions(eps->ds,eps->nconv,0,0,0);CHKERRQ(ierr);
-  ierr = DSSetState(eps->ds,DS_STATE_RAW);CHKERRQ(ierr);
+  ierr = DSTruncate(eps->ds,eps->nconv,PETSC_TRUE);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -557,7 +559,7 @@ static PetscErrorCode EPSKrylovSchurSetPartitions_KrylovSchur(EPS eps,PetscInt n
   if (npart == PETSC_DEFAULT || npart == PETSC_DECIDE) {
     ctx->npart = 1;
   } else {
-    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)eps),&size);CHKERRQ(ierr);
+    ierr = MPI_Comm_size(PetscObjectComm((PetscObject)eps),&size);CHKERRMPI(ierr);
     if (npart<1 || npart>size) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_OUTOFRANGE,"Illegal value of npart");
     ctx->npart = npart;
   }
@@ -1215,7 +1217,7 @@ static PetscErrorCode EPSKrylovSchurUpdateSubcommMats_KrylovSchur(EPS eps,PetscS
         ierr = MatGetOwnershipIS(Ag,&ctx->isrow,&ctx->iscol);CHKERRQ(ierr);
         reuse = PETSC_FALSE;
       }
-      if (str==DIFFERENT_NONZERO_PATTERN) reuse = PETSC_FALSE;
+      if (str==DIFFERENT_NONZERO_PATTERN || str==UNKNOWN_NONZERO_PATTERN) reuse = PETSC_FALSE;
       if (ctx->submata && !reuse) {
         ierr = MatDestroyMatrices(1,&ctx->submata);CHKERRQ(ierr);
       }
@@ -1292,13 +1294,161 @@ PetscErrorCode EPSKrylovSchurUpdateSubcommMats(EPS eps,PetscScalar s,PetscScalar
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode EPSKrylovSchurGetChildEPS(EPS eps,EPS *child)
+{
+  PetscErrorCode   ierr;
+  EPS_KRYLOVSCHUR  *ctx=(EPS_KRYLOVSCHUR*)eps->data,*ctx_local;
+  Mat              A,B=NULL,Ar=NULL,Br=NULL;
+  PetscMPIInt      rank;
+  PetscObjectState Astate,Bstate=0;
+  PetscObjectId    Aid,Bid=0;
+  STType           sttype;
+  PetscInt         nmat;
+  const char       *prefix;
+
+  PetscFunctionBegin;
+  ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
+  if (ctx->npart==1) {
+    if (!ctx->eps) {ierr = EPSCreate(((PetscObject)eps)->comm,&ctx->eps);CHKERRQ(ierr);}
+    ierr = EPSGetOptionsPrefix(eps,&prefix);CHKERRQ(ierr);
+    ierr = EPSSetOptionsPrefix(ctx->eps,prefix);CHKERRQ(ierr);
+    ierr = EPSSetOperators(ctx->eps,A,B);CHKERRQ(ierr);
+  } else {
+    ierr = PetscObjectStateGet((PetscObject)A,&Astate);CHKERRQ(ierr);
+    ierr = PetscObjectGetId((PetscObject)A,&Aid);CHKERRQ(ierr);
+    if (B) {
+      ierr = PetscObjectStateGet((PetscObject)B,&Bstate);CHKERRQ(ierr);
+      ierr = PetscObjectGetId((PetscObject)B,&Bid);CHKERRQ(ierr);
+    }
+    if (!ctx->subc) {
+      /* Create context for subcommunicators */
+      ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)eps),&ctx->subc);CHKERRQ(ierr);
+      ierr = PetscSubcommSetNumber(ctx->subc,ctx->npart);CHKERRQ(ierr);CHKERRQ(ierr);
+      ierr = PetscSubcommSetType(ctx->subc,PETSC_SUBCOMM_CONTIGUOUS);CHKERRQ(ierr);
+      ierr = PetscLogObjectMemory((PetscObject)eps,sizeof(PetscSubcomm));CHKERRQ(ierr);
+
+      /* Duplicate matrices */
+      ierr = MatCreateRedundantMatrix(A,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
+      ctx->Astate = Astate;
+      ctx->Aid = Aid;
+      ierr = MatPropagateSymmetryOptions(A,Ar);CHKERRQ(ierr);
+      if (B) {
+        ierr = MatCreateRedundantMatrix(B,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Br);CHKERRQ(ierr);
+        ctx->Bstate = Bstate;
+        ctx->Bid = Bid;
+        ierr = MatPropagateSymmetryOptions(B,Br);CHKERRQ(ierr);
+      }
+    } else {
+      if (ctx->Astate != Astate || (B && ctx->Bstate != Bstate) || ctx->Aid != Aid || (B && ctx->Bid != Bid)) {
+        ierr = STGetNumMatrices(ctx->eps->st,&nmat);CHKERRQ(ierr);
+        if (nmat) {ierr = EPSGetOperators(ctx->eps,&Ar,&Br);CHKERRQ(ierr);}
+        ierr = MatCreateRedundantMatrix(A,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
+        ctx->Astate = Astate;
+        ctx->Aid = Aid;
+        ierr = MatPropagateSymmetryOptions(A,Ar);CHKERRQ(ierr);
+        if (B) {
+          ierr = MatCreateRedundantMatrix(B,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Br);CHKERRQ(ierr);
+          ctx->Bstate = Bstate;
+          ctx->Bid = Bid;
+          ierr = MatPropagateSymmetryOptions(B,Br);CHKERRQ(ierr);
+        }
+        ierr = EPSSetOperators(ctx->eps,Ar,Br);CHKERRQ(ierr);
+        ierr = MatDestroy(&Ar);CHKERRQ(ierr);
+        ierr = MatDestroy(&Br);CHKERRQ(ierr);
+      }
+    }
+
+    /* Create auxiliary EPS */
+    if (!ctx->eps) {
+      ierr = EPSCreate(PetscSubcommChild(ctx->subc),&ctx->eps);CHKERRQ(ierr);
+      ierr = EPSGetOptionsPrefix(eps,&prefix);CHKERRQ(ierr);
+      ierr = EPSSetOptionsPrefix(ctx->eps,prefix);CHKERRQ(ierr);
+      ierr = EPSSetOperators(ctx->eps,Ar,Br);CHKERRQ(ierr);
+      ierr = MatDestroy(&Ar);CHKERRQ(ierr);
+      ierr = MatDestroy(&Br);CHKERRQ(ierr);
+    }
+    /* Create subcommunicator grouping processes with same rank */
+    if (ctx->commset) { ierr = MPI_Comm_free(&ctx->commrank);CHKERRMPI(ierr); }
+    ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRMPI(ierr);
+    ierr = MPI_Comm_split(((PetscObject)eps)->comm,rank,ctx->subc->color,&ctx->commrank);CHKERRMPI(ierr);
+    ctx->commset = PETSC_TRUE;
+  }
+  ierr = EPSSetType(ctx->eps,((PetscObject)eps)->type_name);CHKERRQ(ierr);
+  ierr = STGetType(eps->st,&sttype);CHKERRQ(ierr);
+  ierr = STSetType(ctx->eps->st,sttype);CHKERRQ(ierr);
+
+  ctx_local = (EPS_KRYLOVSCHUR*)ctx->eps->data;
+  ctx_local->npart = ctx->npart;
+  ctx_local->global = PETSC_FALSE;
+  ctx_local->eps = eps;
+  ctx_local->subc = ctx->subc;
+  ctx_local->commrank = ctx->commrank;
+  *child = ctx->eps;
+  PetscFunctionReturn(0);
+}
+
+static PetscErrorCode EPSKrylovSchurGetKSP_KrylovSchur(EPS eps,KSP *ksp)
+{
+  PetscErrorCode  ierr;
+  EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
+  ST              st;
+  PetscBool       isfilt;
+
+  PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilt);CHKERRQ(ierr);
+  if (eps->which!=EPS_ALL || isfilt) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONGSTATE,"Only available in interval computations with spectrum slicing");
+  ierr = EPSKrylovSchurGetChildEPS(eps,&ctx->eps);CHKERRQ(ierr);
+  ierr = EPSGetST(ctx->eps,&st);CHKERRQ(ierr);
+  ierr = STGetOperator(st,NULL);CHKERRQ(ierr);
+  ierr = STGetKSP(st,ksp);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+/*@
+   EPSKrylovSchurGetKSP - Retrieve the linear solver object associated with the
+   internal EPS object in case of doing spectrum slicing for a computational interval.
+
+   Collective on eps
+
+   Input Parameter:
+.  eps - the eigenproblem solver context
+
+   Output Parameters:
+-  ksp - the internal KSP object
+
+   Notes:
+   When invoked to compute all eigenvalues in an interval with spectrum
+   slicing, EPSKRYLOVSCHUR creates another EPS object internally that is
+   used to compute eigenvalues by chunks near selected shifts. This function
+   allows access to the KSP object associated to this internal EPS object.
+
+   This function is only available for spectrum slicing runs. In case of
+   having more than one partition, the returned KSP will be different
+   in MPI processes belonging to different partitions. Hence, if required,
+   EPSKrylovSchurSetPartitions() must be called BEFORE this function.
+
+   Level: advanced
+
+.seealso: EPSSetInterval(), EPSKrylovSchurSetPartitions()
+@*/
+PetscErrorCode EPSKrylovSchurGetKSP(EPS eps,KSP *ksp)
+{
+  PetscErrorCode ierr;
+
+  PetscFunctionBegin;
+  PetscValidHeaderSpecific(eps,EPS_CLASSID,1);
+  ierr = PetscUseMethod(eps,"EPSKrylovSchurGetKSP_C",(EPS,KSP*),(eps,ksp));CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 PetscErrorCode EPSSetFromOptions_KrylovSchur(PetscOptionItems *PetscOptionsObject,EPS eps)
 {
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
-  PetscBool       flg,lock,b,f1,f2,f3;
+  PetscBool       flg,lock,b,f1,f2,f3,isfilt;
   PetscReal       keep;
   PetscInt        i,j,k;
+  KSP             ksp;
 
   PetscFunctionBegin;
   ierr = PetscOptionsHead(PetscOptionsObject,"EPS Krylov-Schur Options");CHKERRQ(ierr);
@@ -1325,6 +1475,18 @@ PetscErrorCode EPSSetFromOptions_KrylovSchur(PetscOptionItems *PetscOptionsObjec
     if (f1 || f2 || f3) { ierr = EPSKrylovSchurSetDimensions(eps,i,j,k);CHKERRQ(ierr); }
 
   ierr = PetscOptionsTail();CHKERRQ(ierr);
+
+  /* set options of child KSP in spectrum slicing */
+  if (eps->which==EPS_ALL) {
+    if (!eps->st) { ierr = EPSGetST(eps,&eps->st);CHKERRQ(ierr); }
+    ierr = EPSSetDefaultST(eps);CHKERRQ(ierr);
+    ierr = STSetFromOptions(eps->st);CHKERRQ(ierr);  /* need to advance this to check ST type */
+    ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilt);CHKERRQ(ierr);
+    if (!isfilt) {
+      ierr = EPSKrylovSchurGetKSP_KrylovSchur(eps,&ksp);CHKERRQ(ierr);
+      ierr = KSPSetFromOptions(ksp);CHKERRQ(ierr);
+    }
+  }
   PetscFunctionReturn(0);
 }
 
@@ -1333,6 +1495,8 @@ PetscErrorCode EPSView_KrylovSchur(EPS eps,PetscViewer viewer)
   PetscErrorCode  ierr;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data;
   PetscBool       isascii,isfilt;
+  KSP             ksp;
+  PetscViewer     sviewer;
 
   PetscFunctionBegin;
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
@@ -1349,6 +1513,23 @@ PetscErrorCode EPSView_KrylovSchur(EPS eps,PetscViewer viewer)
           ierr = PetscViewerASCIIPrintf(viewer,"  multi-communicator spectrum slicing with %D partitions\n",ctx->npart);CHKERRQ(ierr);
           if (ctx->detect) { ierr = PetscViewerASCIIPrintf(viewer,"  detecting zeros when factorizing at subinterval boundaries\n");CHKERRQ(ierr); }
         }
+        /* view child KSP */
+        ierr = EPSKrylovSchurGetKSP_KrylovSchur(eps,&ksp);CHKERRQ(ierr);
+        ierr = PetscViewerASCIIPushTab(viewer);CHKERRQ(ierr);
+        if (ctx->npart>1 && ctx->subc) {
+          ierr = PetscViewerGetSubViewer(viewer,ctx->subc->child,&sviewer);CHKERRQ(ierr);
+          if (!ctx->subc->color) {
+            ierr = KSPView(ksp,sviewer);CHKERRQ(ierr);
+          }
+          ierr = PetscViewerFlush(sviewer);CHKERRQ(ierr);
+          ierr = PetscViewerRestoreSubViewer(viewer,ctx->subc->child,&sviewer);CHKERRQ(ierr);
+          ierr = PetscViewerFlush(viewer);CHKERRQ(ierr);
+          /* extra call needed because of the two calls to PetscViewerASCIIPushSynchronized() in PetscViewerGetSubViewer() */
+          ierr = PetscViewerASCIIPopSynchronized(viewer);CHKERRQ(ierr);
+        } else {
+          ierr = KSPView(ksp,viewer);CHKERRQ(ierr);
+        }
+        ierr = PetscViewerASCIIPopTab(viewer);CHKERRQ(ierr);
       }
     }
   }
@@ -1358,8 +1539,13 @@ PetscErrorCode EPSView_KrylovSchur(EPS eps,PetscViewer viewer)
 PetscErrorCode EPSDestroy_KrylovSchur(EPS eps)
 {
   PetscErrorCode ierr;
+  PetscBool      isfilt;
 
   PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STFILTER,&isfilt);CHKERRQ(ierr);
+  if (eps->which==EPS_ALL && !isfilt) {
+    ierr = EPSDestroy_KrylovSchur_Slice(eps);CHKERRQ(ierr);
+  }
   ierr = PetscFree(eps->data);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurSetRestart_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetRestart_C",NULL);CHKERRQ(ierr);
@@ -1378,6 +1564,7 @@ PetscErrorCode EPSDestroy_KrylovSchur(EPS eps)
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetSubcommPairs_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetSubcommMats_C",NULL);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurUpdateSubcommMats_C",NULL);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetKSP_C",NULL);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -1452,6 +1639,7 @@ SLEPC_EXTERN PetscErrorCode EPSCreate_KrylovSchur(EPS eps)
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetSubcommPairs_C",EPSKrylovSchurGetSubcommPairs_KrylovSchur);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetSubcommMats_C",EPSKrylovSchurGetSubcommMats_KrylovSchur);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurUpdateSubcommMats_C",EPSKrylovSchurUpdateSubcommMats_KrylovSchur);CHKERRQ(ierr);
+  ierr = PetscObjectComposeFunction((PetscObject)eps,"EPSKrylovSchurGetKSP_C",EPSKrylovSchurGetKSP_KrylovSchur);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
