@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -81,27 +81,38 @@ PetscErrorCode EPSReset_KrylovSchur_Slice(EPS eps)
 
   PetscFunctionBegin;
   if (!ctx->global) PetscFunctionReturn(0);
-  /* Destroy auxiliary EPS */
+  /* Reset auxiliary EPS */
   ierr = EPSSliceResetSR(ctx->eps);CHKERRQ(ierr);
+  ierr = EPSReset(ctx->eps);CHKERRQ(ierr);
+  ierr = EPSSliceResetSR(eps);CHKERRQ(ierr);
+  ierr = PetscFree(ctx->inertias);CHKERRQ(ierr);
+  ierr = PetscFree(ctx->shifts);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode EPSDestroy_KrylovSchur_Slice(EPS eps)
+{
+  PetscErrorCode  ierr;
+  EPS_KRYLOVSCHUR *ctx=(EPS_KRYLOVSCHUR*)eps->data;
+
+  PetscFunctionBegin;
+  if (!ctx->global) PetscFunctionReturn(0);
+  /* Destroy auxiliary EPS */
+  ierr = EPSReset_KrylovSchur_Slice(eps);CHKERRQ(ierr);
   ierr = EPSDestroy(&ctx->eps);CHKERRQ(ierr);
   if (ctx->npart>1) {
     ierr = PetscSubcommDestroy(&ctx->subc);CHKERRQ(ierr);
     if (ctx->commset) {
-      ierr = MPI_Comm_free(&ctx->commrank);CHKERRQ(ierr);
+      ierr = MPI_Comm_free(&ctx->commrank);CHKERRMPI(ierr);
       ctx->commset = PETSC_FALSE;
     }
-  }
-  ierr = PetscFree(ctx->subintervals);CHKERRQ(ierr);
-  ierr = PetscFree(ctx->nconv_loc);CHKERRQ(ierr);
-  ierr = EPSSliceResetSR(eps);CHKERRQ(ierr);
-  ierr = PetscFree(ctx->inertias);CHKERRQ(ierr);
-  ierr = PetscFree(ctx->shifts);CHKERRQ(ierr);
-  if (ctx->npart>1) {
     ierr = ISDestroy(&ctx->isrow);CHKERRQ(ierr);
     ierr = ISDestroy(&ctx->iscol);CHKERRQ(ierr);
     ierr = MatDestroyMatrices(1,&ctx->submata);CHKERRQ(ierr);
     ierr = MatDestroyMatrices(1,&ctx->submatb);CHKERRQ(ierr);
   }
+  ierr = PetscFree(ctx->subintervals);CHKERRQ(ierr);
+  ierr = PetscFree(ctx->nconv_loc);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -165,68 +176,18 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
   BVOrthogType       orthog_type;
   BVOrthogRefineType orthog_ref;
   BVOrthogBlockType  ob_type;
-  Mat                A,B=NULL,Ar=NULL,Br=NULL;
   PetscInt           i;
-  PetscReal          h,a,b,zero;
-  PetscMPIInt        rank;
+  PetscReal          h,a,b;
+  PetscRandom        rand;
   EPS_SR             sr=ctx->sr;
-  PC                 pc;
-  PCType             pctype;
-  KSP                ksp;
-  KSPType            ksptype;
-  STType             sttype;
-  PetscObjectState   Astate,Bstate=0;
-  PetscObjectId      Aid,Bid=0;
-  MatSolverType      stype;
 
   PetscFunctionBegin;
-  ierr = EPSGetOperators(eps,&A,&B);CHKERRQ(ierr);
+  if (!ctx->eps) { ierr = EPSKrylovSchurGetChildEPS(eps,&ctx->eps);CHKERRQ(ierr); }
+
+  /* Determine subintervals */
   if (ctx->npart==1) {
-    if (!ctx->eps) { ierr = EPSCreate(((PetscObject)eps)->comm,&ctx->eps);CHKERRQ(ierr); }
-    ierr = EPSSetType(ctx->eps,((PetscObject)eps)->type_name);CHKERRQ(ierr);
-    ierr = EPSSetOperators(ctx->eps,A,B);CHKERRQ(ierr);
     a = eps->inta; b = eps->intb;
   } else {
-    ierr = PetscObjectStateGet((PetscObject)A,&Astate);CHKERRQ(ierr);
-    ierr = PetscObjectGetId((PetscObject)A,&Aid);CHKERRQ(ierr);
-    if (B) {
-      ierr = PetscObjectStateGet((PetscObject)B,&Bstate);CHKERRQ(ierr);
-      ierr = PetscObjectGetId((PetscObject)B,&Bid);CHKERRQ(ierr);
-    }
-    if (!ctx->subc) {
-      /* Create context for subcommunicators */
-      ierr = PetscSubcommCreate(PetscObjectComm((PetscObject)eps),&ctx->subc);CHKERRQ(ierr);
-      ierr = PetscSubcommSetNumber(ctx->subc,ctx->npart);CHKERRQ(ierr);CHKERRQ(ierr);
-      ierr = PetscSubcommSetType(ctx->subc,PETSC_SUBCOMM_CONTIGUOUS);CHKERRQ(ierr);
-      ierr = PetscLogObjectMemory((PetscObject)eps,sizeof(PetscSubcomm));CHKERRQ(ierr);
-
-      /* Duplicate matrices */
-      ierr = MatCreateRedundantMatrix(A,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
-      ctx->Astate = Astate;
-      ctx->Aid = Aid;
-      if (B) {
-        ierr = MatCreateRedundantMatrix(B,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Br);CHKERRQ(ierr);
-        ctx->Bstate = Bstate;
-        ctx->Bid = Bid;
-      }
-    } else {
-      if (ctx->Astate != Astate || (B && ctx->Bstate != Bstate) || ctx->Aid != Aid || (B && ctx->Bid != Bid)) {
-        ierr = EPSGetOperators(ctx->eps,&Ar,&Br);CHKERRQ(ierr);
-        ierr = MatCreateRedundantMatrix(A,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Ar);CHKERRQ(ierr);
-        ctx->Astate = Astate;
-        ctx->Aid = Aid;
-        if (B) {
-          ierr = MatCreateRedundantMatrix(B,0,PetscSubcommChild(ctx->subc),MAT_INITIAL_MATRIX,&Br);CHKERRQ(ierr);
-          ctx->Bstate = Bstate;
-          ctx->Bid = Bid;
-        }
-        ierr = EPSSetOperators(ctx->eps,Ar,Br);CHKERRQ(ierr);
-        ierr = MatDestroy(&Ar);CHKERRQ(ierr);
-        ierr = MatDestroy(&Br);CHKERRQ(ierr);
-      }
-    }
-
-    /* Determine subintervals */
     if (!ctx->subintset) { /* uniform distribution if no set by user */
       if (!sr->hasEnd) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"Global interval must be bounded for splitting it in uniform subintervals");
       h = (eps->intb-eps->inta)/ctx->npart;
@@ -240,54 +201,18 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
       a = ctx->subintervals[ctx->subc->color];
       b = ctx->subintervals[ctx->subc->color+1];
     }
-
-    if (!ctx->eps) {
-      /* Create auxiliary EPS */
-      ierr = EPSCreate(PetscSubcommChild(ctx->subc),&ctx->eps);CHKERRQ(ierr);
-      ierr = EPSSetOperators(ctx->eps,Ar,Br);CHKERRQ(ierr);
-      ierr = MatDestroy(&Ar);CHKERRQ(ierr);
-      ierr = MatDestroy(&Br);CHKERRQ(ierr);
-    }
-
-    /* Create subcommunicator grouping processes with same rank */
-    if (ctx->commset) { ierr = MPI_Comm_free(&ctx->commrank);CHKERRQ(ierr); }
-    ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
-    ierr = MPI_Comm_split(((PetscObject)eps)->comm,rank,ctx->subc->color,&ctx->commrank);CHKERRQ(ierr);
-    ctx->commset = PETSC_TRUE;
   }
-  ierr = EPSSetType(ctx->eps,((PetscObject)eps)->type_name);CHKERRQ(ierr);
-
-  /* Transfer options for ST, KSP and PC */
-  ierr = STGetType(eps->st,&sttype);CHKERRQ(ierr);
-  ierr = STSetType(ctx->eps->st,sttype);CHKERRQ(ierr);
-  ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
-  ierr = KSPGetType(ksp,&ksptype);CHKERRQ(ierr);
-  ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-  ierr = PCGetType(pc,&pctype);CHKERRQ(ierr);
-  ierr = PCFactorGetMatSolverType(pc,&stype);CHKERRQ(ierr);
-  ierr = PCFactorGetZeroPivot(pc,&zero);CHKERRQ(ierr);
-  ierr = STGetKSP(ctx->eps->st,&ksp);CHKERRQ(ierr);
-  ierr = KSPSetType(ksp,ksptype);CHKERRQ(ierr);
-  ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
-  ierr = PCSetType(pc,pctype);CHKERRQ(ierr);
-  ierr = PCFactorSetZeroPivot(pc,zero);CHKERRQ(ierr);
-  if (stype) { ierr = PCFactorSetMatSolverType(pc,stype);CHKERRQ(ierr); }
-
-  ierr = EPSSetConvergenceTest(ctx->eps,eps->conv);CHKERRQ(ierr);
   ierr = EPSSetInterval(ctx->eps,a,b);CHKERRQ(ierr);
-  ctx_local = (EPS_KRYLOVSCHUR*)ctx->eps->data;
-  ctx_local->npart = ctx->npart;
-  ctx_local->detect = ctx->detect;
-  ctx_local->global = PETSC_FALSE;
-  ctx_local->eps = eps;
-  ctx_local->subc = ctx->subc;
-  ctx_local->commrank = ctx->commrank;
-
+  ierr = EPSSetConvergenceTest(ctx->eps,eps->conv);CHKERRQ(ierr);
   ierr = EPSSetDimensions(ctx->eps,ctx->nev,ctx->ncv,ctx->mpd);CHKERRQ(ierr);
   ierr = EPSKrylovSchurSetLocking(ctx->eps,ctx->lock);CHKERRQ(ierr);
 
+  ctx_local = (EPS_KRYLOVSCHUR*)ctx->eps->data;
+  ctx_local->detect = ctx->detect;
+
   /* transfer options from eps->V */
   ierr = EPSGetBV(ctx->eps,&V);CHKERRQ(ierr);
+  ierr = BVGetRandomContext(V,&rand);CHKERRQ(ierr);  /* make sure the random context is available when duplicating */
   if (!eps->V) { ierr = EPSGetBV(eps,&eps->V);CHKERRQ(ierr); }
   if (!((PetscObject)(eps->V))->type_name) {
     ierr = BVSetType(V,BVSVEC);CHKERRQ(ierr);
@@ -297,6 +222,7 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
   }
   ierr = BVGetOrthogonalization(eps->V,&orthog_type,&orthog_ref,&eta,&ob_type);CHKERRQ(ierr);
   ierr = BVSetOrthogonalization(V,orthog_type,orthog_ref,eta,ob_type);CHKERRQ(ierr);
+
   ctx->eps->which = eps->which;
   ctx->eps->max_it = eps->max_it;
   ctx->eps->tol = eps->tol;
@@ -317,10 +243,11 @@ static PetscErrorCode EPSSliceGetEPS(EPS eps)
 static PetscErrorCode EPSSliceGetInertia(EPS eps,PetscReal shift,PetscInt *inertia,PetscInt *zeros)
 {
   PetscErrorCode ierr;
-  KSP            ksp;
+  KSP            ksp,kspr;
   PC             pc;
   Mat            F;
   PetscReal      nzshift=shift;
+  PetscBool      flg;
 
   PetscFunctionBegin;
   if (shift >= PETSC_MAX_REAL) { /* Right-open interval */
@@ -336,6 +263,11 @@ static PetscErrorCode EPSSliceGetInertia(EPS eps,PetscReal shift,PetscInt *inert
     ierr = STSetShift(eps->st,nzshift);CHKERRQ(ierr);
     ierr = STGetKSP(eps->st,&ksp);CHKERRQ(ierr);
     ierr = KSPGetPC(ksp,&pc);CHKERRQ(ierr);
+    ierr = PetscObjectTypeCompare((PetscObject)pc,PCREDUNDANT,&flg);CHKERRQ(ierr);
+    if (flg) {
+      ierr = PCRedundantGetKSP(pc,&kspr);CHKERRQ(ierr);
+      ierr = KSPGetPC(kspr,&pc);CHKERRQ(ierr);
+    }
     ierr = PCFactorGetMatrix(pc,&F);CHKERRQ(ierr);
     ierr = MatGetInertia(F,inertia,zeros,NULL);CHKERRQ(ierr);
   }
@@ -355,7 +287,6 @@ static PetscErrorCode EPSBackTransform_Skip(EPS eps)
 PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
 {
   PetscErrorCode  ierr;
-  PetscBool       issinv;
   EPS_KRYLOVSCHUR *ctx = (EPS_KRYLOVSCHUR*)eps->data,*ctx_glob;
   EPS_SR          sr,sr_loc,sr_glob;
   PetscInt        nEigs,dssz=1,i,zeros=0,off=0,method,hiteig=0;
@@ -363,18 +294,25 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
   PetscReal       r;
   MPI_Request     req;
   Mat             A,B=NULL;
-  SlepcSC         sc;
   DSParallelType  ptype;
 
   PetscFunctionBegin;
   if (ctx->global) {
+    EPSCheckHermitianDefiniteCondition(eps,PETSC_TRUE," with spectrum slicing");
+    EPSCheckSinvertCayleyCondition(eps,PETSC_TRUE," with spectrum slicing");
+    if (eps->inta==eps->intb) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support computing all eigenvalues unless you provide a computational interval with EPSSetInterval()");
     if (eps->intb >= PETSC_MAX_REAL && eps->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"The defined computational interval should have at least one of their sides bounded");
-    if (!eps->ishermitian) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Spectrum slicing only available for symmetric/Hermitian eigenproblems");
-    if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs cannot be used with spectrum slicing");
-    ierr = PetscObjectTypeCompareAny((PetscObject)eps->st,&issinv,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
-    if (!issinv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Shift-and-invert or Cayley ST is needed for spectrum slicing");
-    if (eps->tol==PETSC_DEFAULT) eps->tol = SLEPC_DEFAULT_TOL*1e-2;  /* use tighter tolerance */
-    if (!eps->max_it) eps->max_it = 100;
+    EPSCheckUnsupportedCondition(eps,EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_STOPPING,PETSC_TRUE," with spectrum slicing");
+    EPSCheckIgnoredCondition(eps,EPS_FEATURE_BALANCE,PETSC_TRUE," with spectrum slicing");
+    if (eps->tol==PETSC_DEFAULT) {
+ #if defined(PETSC_USE_REAL_SINGLE)
+      eps->tol = SLEPC_DEFAULT_TOL;
+#else
+      /* use tighter tolerance */
+      eps->tol = SLEPC_DEFAULT_TOL*1e-2;
+#endif
+    }
+    if (eps->max_it==PETSC_DEFAULT) eps->max_it = 100;
     if (ctx->nev==1) ctx->nev = PetscMin(40,eps->n);  /* nev not set, use default value */
     if (eps->n>10 && ctx->nev<10) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_ARG_WRONG,"nev cannot be less than 10 in spectrum slicing runs");
   }
@@ -408,37 +346,38 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     }
   }
   if (ctx->global) {
-    /* prevent computation of factorization in global eps */
-    ierr = STSetTransform(eps->st,PETSC_FALSE);CHKERRQ(ierr);
     ierr = EPSSetDimensions_Default(eps,ctx->nev,&ctx->ncv,&ctx->mpd);CHKERRQ(ierr);
     /* create subintervals and initialize auxiliary eps for slicing runs */
+    ierr = EPSKrylovSchurGetChildEPS(eps,&ctx->eps);CHKERRQ(ierr);
+    /* prevent computation of factorization in global eps */
+    ierr = STSetTransform(eps->st,PETSC_FALSE);CHKERRQ(ierr);
     ierr = EPSSliceGetEPS(eps);CHKERRQ(ierr);
     sr_loc = ((EPS_KRYLOVSCHUR*)ctx->eps->data)->sr;
     if (ctx->npart>1) {
       if ((sr->dir>0&&ctx->subc->color==0)||(sr->dir<0&&ctx->subc->color==ctx->npart-1)) sr->inertia0 = sr_loc->inertia0;
-      ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
+      ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRMPI(ierr);
       if (!rank) {
-        ierr = MPI_Bcast(&sr->inertia0,1,MPIU_INT,(sr->dir>0)?0:ctx->npart-1,ctx->commrank);CHKERRQ(ierr);
+        ierr = MPI_Bcast(&sr->inertia0,1,MPIU_INT,(sr->dir>0)?0:ctx->npart-1,ctx->commrank);CHKERRMPI(ierr);
       }
-      ierr = MPI_Bcast(&sr->inertia0,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
+      ierr = MPI_Bcast(&sr->inertia0,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
       ierr = PetscFree(ctx->nconv_loc);CHKERRQ(ierr);
       ierr = PetscMalloc1(ctx->npart,&ctx->nconv_loc);CHKERRQ(ierr);
-      ierr = MPI_Comm_size(((PetscObject)eps)->comm,&nproc);CHKERRQ(ierr);
+      ierr = MPI_Comm_size(((PetscObject)eps)->comm,&nproc);CHKERRMPI(ierr);
       if (sr->dir<0) off = 1;
       if (nproc%ctx->npart==0) { /* subcommunicators with the same size */
         ierr = PetscMPIIntCast(sr_loc->numEigs,&aux);CHKERRQ(ierr);
-        ierr = MPI_Allgather(&aux,1,MPI_INT,ctx->nconv_loc,1,MPI_INT,ctx->commrank);CHKERRQ(ierr);
-        ierr = MPI_Allgather(sr_loc->dir==sr->dir?&sr_loc->int0:&sr_loc->int1,1,MPIU_REAL,ctx->subintervals+off,1,MPIU_REAL,ctx->commrank);CHKERRQ(ierr);
+        ierr = MPI_Allgather(&aux,1,MPI_INT,ctx->nconv_loc,1,MPI_INT,ctx->commrank);CHKERRMPI(ierr);
+        ierr = MPI_Allgather(sr_loc->dir==sr->dir?&sr_loc->int0:&sr_loc->int1,1,MPIU_REAL,ctx->subintervals+off,1,MPIU_REAL,ctx->commrank);CHKERRMPI(ierr);
       } else {
-        ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
+        ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRMPI(ierr);
         if (!rank) {
           ierr = PetscMPIIntCast(sr_loc->numEigs,&aux);CHKERRQ(ierr);
-          ierr = MPI_Allgather(&aux,1,MPI_INT,ctx->nconv_loc,1,MPI_INT,ctx->commrank);CHKERRQ(ierr);
-          ierr = MPI_Allgather(sr_loc->dir==sr->dir?&sr_loc->int0:&sr_loc->int1,1,MPIU_REAL,ctx->subintervals+off,1,MPIU_REAL,ctx->commrank);CHKERRQ(ierr);
+          ierr = MPI_Allgather(&aux,1,MPI_INT,ctx->nconv_loc,1,MPI_INT,ctx->commrank);CHKERRMPI(ierr);
+          ierr = MPI_Allgather(sr_loc->dir==sr->dir?&sr_loc->int0:&sr_loc->int1,1,MPIU_REAL,ctx->subintervals+off,1,MPIU_REAL,ctx->commrank);CHKERRMPI(ierr);
         }
         ierr = PetscMPIIntCast(ctx->npart,&aux);CHKERRQ(ierr);
-        ierr = MPI_Bcast(ctx->nconv_loc,aux,MPI_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
-        ierr = MPI_Bcast(ctx->subintervals+off,aux,MPIU_REAL,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
+        ierr = MPI_Bcast(ctx->nconv_loc,aux,MPI_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
+        ierr = MPI_Bcast(ctx->subintervals+off,aux,MPIU_REAL,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
       }
       nEigs = 0;
       for (i=0;i<ctx->npart;i++) nEigs += ctx->nconv_loc[i];
@@ -462,6 +401,8 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
       if ((sr->dir>0&&ctx->subc->color==ctx->npart-1)||(sr->dir<0&&ctx->subc->color==0)) sr->hasEnd = sr_glob->hasEnd;
       else sr->hasEnd = PETSC_TRUE;
     }
+    /* sets first shift */
+    ierr = STSetShift(eps->st,(sr->int0==0.0)?10.0/PETSC_MAX_REAL:sr->int0);CHKERRQ(ierr);
     ierr = STSetUp(eps->st);CHKERRQ(ierr);
 
     /* compute inertia0 */
@@ -489,25 +430,25 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
     }
     if (ctx->npart>1) {
       /* inertia1 is received from neighbour */
-      ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
+      ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRMPI(ierr);
       if (!rank) {
         if (sr->inertia0!=-1 && ((sr->dir>0 && ctx->subc->color>0) || (sr->dir<0 && ctx->subc->color<ctx->npart-1))) { /* send inertia0 to neighbour0 */
-          ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
-          ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+          ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRMPI(ierr);
+          ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRMPI(ierr);
         }
         if ((sr->dir>0 && ctx->subc->color<ctx->npart-1)|| (sr->dir<0 && ctx->subc->color>0)) { /* receive inertia1 from neighbour1 */
-          ierr = MPI_Recv(&(sr->inertia1),1,MPIU_INT,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRQ(ierr);
-          ierr = MPI_Recv(&(sr->int1),1,MPIU_REAL,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRQ(ierr);
+          ierr = MPI_Recv(&(sr->inertia1),1,MPIU_INT,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRMPI(ierr);
+          ierr = MPI_Recv(&(sr->int1),1,MPIU_REAL,ctx->subc->color+sr->dir,0,ctx->commrank,MPI_STATUS_IGNORE);CHKERRMPI(ierr);
         }
         if (sr->inertia0==-1 && !(sr->dir>0 && ctx->subc->color==ctx->npart-1) && !(sr->dir<0 && ctx->subc->color==0)) {
           sr->inertia0 = sr->inertia1; sr->int0 = sr->int1;
-          ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
-          ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+          ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRMPI(ierr);
+          ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRMPI(ierr);
         }
       }
       if ((sr->dir>0 && ctx->subc->color<ctx->npart-1)||(sr->dir<0 && ctx->subc->color>0)) {
-        ierr = MPI_Bcast(&sr->inertia1,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
-        ierr = MPI_Bcast(&sr->int1,1,MPIU_REAL,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
+        ierr = MPI_Bcast(&sr->inertia1,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
+        ierr = MPI_Bcast(&sr->int1,1,MPIU_REAL,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
       } else sr_glob->inertia1 = sr->inertia1;
     }
 
@@ -517,8 +458,8 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
       if (zeros) SETERRQ(((PetscObject)eps)->comm,PETSC_ERR_USER,"Found singular matrix for the transformed problem in an interval endpoint defined by user");
       if (!rank && sr->inertia0==-1) {
         sr->inertia0 = sr->inertia1; sr->int0 = sr->int1;
-        ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
-        ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRQ(ierr);
+        ierr = MPI_Isend(&(sr->inertia0),1,MPIU_INT,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRMPI(ierr);
+        ierr = MPI_Isend(&(sr->int0),1,MPIU_REAL,ctx->subc->color-sr->dir,0,ctx->commrank,&req);CHKERRMPI(ierr);
       }
       if (sr->hasEnd) {
         sr->dir = -sr->dir; r = sr->int0; sr->int0 = sr->int1; sr->int1 = r;
@@ -533,14 +474,6 @@ PetscErrorCode EPSSetUp_KrylovSchur_Slice(EPS eps)
       ierr = EPSSliceAllocateSolution(eps,1);CHKERRQ(ierr);
     }
     dssz = eps->ncv+1;
-    if (sr->numEigs>0) {
-      ierr = DSGetSlepcSC(eps->ds,&sc);CHKERRQ(ierr);
-      sc->rg            = NULL;
-      sc->comparison    = SlepcCompareLargestMagnitude;
-      sc->comparisonctx = NULL;
-      sc->map           = NULL;
-      sc->mapobj        = NULL;
-    }
     ierr = DSGetParallel(ctx->eps->ds,&ptype);CHKERRQ(ierr);
     ierr = DSSetParallel(eps->ds,ptype);CHKERRQ(ierr);
     ierr = DSGetMethod(ctx->eps->ds,&method);CHKERRQ(ierr);
@@ -725,11 +658,11 @@ static PetscErrorCode EPSSliceGatherSolution(EPS eps)
     }
   }
   ierr = PetscMalloc1(ctx->npart,&ns_loc);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRMPI(ierr);
   ierr = PetscMPIIntCast(ns,&aux);CHKERRQ(ierr);
-  if (!rank) { ierr = MPI_Allgather(&aux,1,MPI_INT,ns_loc,1,MPI_INT,ctx->commrank);CHKERRQ(ierr); }
+  if (!rank) { ierr = MPI_Allgather(&aux,1,MPI_INT,ns_loc,1,MPI_INT,ctx->commrank);CHKERRMPI(ierr); }
   ierr = PetscMPIIntCast(ctx->npart,&aux);CHKERRQ(ierr);
-  ierr = MPI_Bcast(ns_loc,aux,MPI_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
+  ierr = MPI_Bcast(ns_loc,aux,MPI_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
   ctx->nshifts = 0;
   for (i=0;i<ctx->npart;i++) ctx->nshifts += ns_loc[i];
   ierr = PetscFree(ctx->inertias);CHKERRQ(ierr);
@@ -740,38 +673,38 @@ static PetscErrorCode EPSSliceGatherSolution(EPS eps)
   /* Gather eigenvalues (same ranks have fully set of eigenvalues)*/
   eigr_loc = sr_loc->eigr;
   perm_loc = sr_loc->perm;
-  ierr = MPI_Comm_size(((PetscObject)eps)->comm,&nproc);CHKERRQ(ierr);
+  ierr = MPI_Comm_size(((PetscObject)eps)->comm,&nproc);CHKERRMPI(ierr);
   ierr = PetscMalloc1(ctx->npart,&disp);CHKERRQ(ierr);
   disp[0] = 0;
   for (i=1;i<ctx->npart;i++) disp[i] = disp[i-1]+ctx->nconv_loc[i-1];
   if (nproc%ctx->npart==0) { /* subcommunicators with the same size */
     ierr = PetscMPIIntCast(sr_loc->numEigs,&aux);CHKERRQ(ierr);
-    ierr = MPI_Allgatherv(eigr_loc,aux,MPIU_SCALAR,eps->eigr,ctx->nconv_loc,disp,MPIU_SCALAR,ctx->commrank);CHKERRQ(ierr); /* eigenvalues */
-    ierr = MPI_Allgatherv(perm_loc,aux,MPIU_INT,eps->perm,ctx->nconv_loc,disp,MPIU_INT,ctx->commrank);CHKERRQ(ierr); /* perm */
+    ierr = MPI_Allgatherv(eigr_loc,aux,MPIU_SCALAR,eps->eigr,ctx->nconv_loc,disp,MPIU_SCALAR,ctx->commrank);CHKERRMPI(ierr); /* eigenvalues */
+    ierr = MPI_Allgatherv(perm_loc,aux,MPIU_INT,eps->perm,ctx->nconv_loc,disp,MPIU_INT,ctx->commrank);CHKERRMPI(ierr); /* perm */
     for (i=1;i<ctx->npart;i++) disp[i] = disp[i-1]+ns_loc[i-1];
     ierr = PetscMPIIntCast(ns,&aux);CHKERRQ(ierr);
-    ierr = MPI_Allgatherv(shifts_loc,aux,MPIU_REAL,ctx->shifts,ns_loc,disp,MPIU_REAL,ctx->commrank);CHKERRQ(ierr); /* shifts */
-    ierr = MPI_Allgatherv(inertias_loc,aux,MPIU_INT,ctx->inertias,ns_loc,disp,MPIU_INT,ctx->commrank);CHKERRQ(ierr); /* inertias */
-    ierr = MPI_Allreduce(&sr_loc->itsKs,&eps->its,1,MPIU_INT,MPI_SUM,ctx->commrank);CHKERRQ(ierr);
+    ierr = MPI_Allgatherv(shifts_loc,aux,MPIU_REAL,ctx->shifts,ns_loc,disp,MPIU_REAL,ctx->commrank);CHKERRMPI(ierr); /* shifts */
+    ierr = MPI_Allgatherv(inertias_loc,aux,MPIU_INT,ctx->inertias,ns_loc,disp,MPIU_INT,ctx->commrank);CHKERRMPI(ierr); /* inertias */
+    ierr = MPIU_Allreduce(&sr_loc->itsKs,&eps->its,1,MPIU_INT,MPI_SUM,ctx->commrank);CHKERRQ(ierr);
   } else { /* subcommunicators with different size */
-    ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRQ(ierr);
+    ierr = MPI_Comm_rank(PetscSubcommChild(ctx->subc),&rank);CHKERRMPI(ierr);
     if (!rank) {
       ierr = PetscMPIIntCast(sr_loc->numEigs,&aux);CHKERRQ(ierr);
-      ierr = MPI_Allgatherv(eigr_loc,aux,MPIU_SCALAR,eps->eigr,ctx->nconv_loc,disp,MPIU_SCALAR,ctx->commrank);CHKERRQ(ierr); /* eigenvalues */
-      ierr = MPI_Allgatherv(perm_loc,aux,MPIU_INT,eps->perm,ctx->nconv_loc,disp,MPIU_INT,ctx->commrank);CHKERRQ(ierr); /* perm */
+      ierr = MPI_Allgatherv(eigr_loc,aux,MPIU_SCALAR,eps->eigr,ctx->nconv_loc,disp,MPIU_SCALAR,ctx->commrank);CHKERRMPI(ierr); /* eigenvalues */
+      ierr = MPI_Allgatherv(perm_loc,aux,MPIU_INT,eps->perm,ctx->nconv_loc,disp,MPIU_INT,ctx->commrank);CHKERRMPI(ierr); /* perm */
       for (i=1;i<ctx->npart;i++) disp[i] = disp[i-1]+ns_loc[i-1];
       ierr = PetscMPIIntCast(ns,&aux);CHKERRQ(ierr);
-      ierr = MPI_Allgatherv(shifts_loc,aux,MPIU_REAL,ctx->shifts,ns_loc,disp,MPIU_REAL,ctx->commrank);CHKERRQ(ierr); /* shifts */
-      ierr = MPI_Allgatherv(inertias_loc,aux,MPIU_INT,ctx->inertias,ns_loc,disp,MPIU_INT,ctx->commrank);CHKERRQ(ierr); /* inertias */
-      ierr = MPI_Allreduce(&sr_loc->itsKs,&eps->its,1,MPIU_INT,MPI_SUM,ctx->commrank);CHKERRQ(ierr);
+      ierr = MPI_Allgatherv(shifts_loc,aux,MPIU_REAL,ctx->shifts,ns_loc,disp,MPIU_REAL,ctx->commrank);CHKERRMPI(ierr); /* shifts */
+      ierr = MPI_Allgatherv(inertias_loc,aux,MPIU_INT,ctx->inertias,ns_loc,disp,MPIU_INT,ctx->commrank);CHKERRMPI(ierr); /* inertias */
+      ierr = MPIU_Allreduce(&sr_loc->itsKs,&eps->its,1,MPIU_INT,MPI_SUM,ctx->commrank);CHKERRQ(ierr);
     }
     ierr = PetscMPIIntCast(eps->nconv,&aux);CHKERRQ(ierr);
-    ierr = MPI_Bcast(eps->eigr,aux,MPIU_SCALAR,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
-    ierr = MPI_Bcast(eps->perm,aux,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
-    ierr = MPI_Bcast(ctx->shifts,ctx->nshifts,MPIU_REAL,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
+    ierr = MPI_Bcast(eps->eigr,aux,MPIU_SCALAR,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
+    ierr = MPI_Bcast(eps->perm,aux,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
+    ierr = MPI_Bcast(ctx->shifts,ctx->nshifts,MPIU_REAL,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
     ierr = PetscMPIIntCast(ctx->nshifts,&aux);CHKERRQ(ierr);
-    ierr = MPI_Bcast(ctx->inertias,aux,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
-    ierr = MPI_Bcast(&eps->its,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRQ(ierr);
+    ierr = MPI_Bcast(ctx->inertias,aux,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
+    ierr = MPI_Bcast(&eps->its,1,MPIU_INT,0,PetscSubcommChild(ctx->subc));CHKERRMPI(ierr);
   }
   /* Update global array eps->perm */
   idx = ctx->nconv_loc[0];
@@ -933,8 +866,7 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
   EPS_shift       sPres;
   PetscBool       breakdown,complIterating,sch0,sch1;
   EPS_SR          sr = ctx->sr;
-#define MSGLEN 100
-  char            messg[MSGLEN];
+  char            messg[100];
 
   PetscFunctionBegin;
   /* Spectrum slicing data */
@@ -1094,7 +1026,7 @@ static PetscErrorCode EPSKrylovSchur_Slice(EPS eps)
   }
   sPres->comp[0] = PetscNot(count0 < sPres->nsch[0]);
   sPres->comp[1] = PetscNot(count1 < sPres->nsch[1]);
-  ierr = PetscSNPrintf(messg,MSGLEN,"Lanczos: %D evals in [%g,%g]%s and %D evals in [%g,%g]%s\n",count0,(double)(sr->dir==1)?sPres->ext[0]:sPres->value,(double)(sr->dir==1)?sPres->value:sPres->ext[0],(sPres->comp[0])?"*":"",count1,(double)(sr->dir==1)?sPres->value:sPres->ext[1],(double)(sr->dir==1)?sPres->ext[1]:sPres->value,(sPres->comp[1])?"*":"");CHKERRQ(ierr);
+  ierr = PetscSNPrintf(messg,sizeof(messg),"Lanczos: %D evals in [%g,%g]%s and %D evals in [%g,%g]%s\n",count0,(double)(sr->dir==1)?sPres->ext[0]:sPres->value,(double)(sr->dir==1)?sPres->value:sPres->ext[0],(sPres->comp[0])?"*":"",count1,(double)(sr->dir==1)?sPres->value:sPres->ext[1],(double)(sr->dir==1)?sPres->ext[1]:sPres->value,(sPres->comp[1])?"*":"");CHKERRQ(ierr);
   ierr = PetscInfo(eps,messg);CHKERRQ(ierr);
   if (count0 > sPres->nsch[0] || count1 > sPres->nsch[1]) InertiaMismatch(eps,ctx->detect);
   ierr = PetscFree(iwork);CHKERRQ(ierr);
