@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -55,28 +55,22 @@ typedef struct {
 PetscErrorCode EPSSetUp_LyapII(EPS eps)
 {
   PetscErrorCode ierr;
+  PetscRandom    rand;
   EPS_LYAPII     *ctx = (EPS_LYAPII*)eps->data;
-  PetscBool      istrivial,issinv;
 
   PetscFunctionBegin;
-  if (eps->ncv) {
+  EPSCheckSinvert(eps);
+  if (eps->ncv!=PETSC_DEFAULT) {
     if (eps->ncv<eps->nev+1) SETERRQ(PetscObjectComm((PetscObject)eps),1,"The value of ncv must be at least nev+1");
   } else eps->ncv = eps->nev+1;
-  if (eps->mpd) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
+  if (eps->mpd!=PETSC_DEFAULT) { ierr = PetscInfo(eps,"Warning: parameter mpd ignored\n");CHKERRQ(ierr); }
+  if (eps->max_it==PETSC_DEFAULT) eps->max_it = PetscMax(1000*eps->nev,100*eps->n);
+  if (!eps->which) eps->which=EPS_LARGEST_REAL;
+  if (eps->which!=EPS_LARGEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver supports only largest real eigenvalues");
+  EPSCheckUnsupported(eps,EPS_FEATURE_BALANCE | EPS_FEATURE_ARBITRARY | EPS_FEATURE_REGION | EPS_FEATURE_EXTRACTION | EPS_FEATURE_TWOSIDED);
+
   if (!ctx->rkc) ctx->rkc = 10;
   if (!ctx->rkl) ctx->rkl = 3*ctx->rkc;
-  if (!eps->max_it) eps->max_it = PetscMax(1000*eps->nev,100*eps->n);
-  if (!eps->which) eps->which=EPS_LARGEST_REAL;
-  if (eps->which!=EPS_LARGEST_REAL) SETERRQ(PetscObjectComm((PetscObject)eps),1,"Wrong value of eps->which");
-  if (eps->extraction) { ierr = PetscInfo(eps,"Warning: extraction type ignored\n");CHKERRQ(ierr); }
-  if (eps->balance!=EPS_BALANCE_NONE) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Balancing not supported in this solver");
-  if (eps->arbitrary) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Arbitrary selection of eigenpairs not supported in this solver");
-  ierr = RGIsTrivial(eps->rg,&istrivial);CHKERRQ(ierr);
-  if (!istrivial) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"This solver does not support region filtering");
-
-  ierr = PetscObjectTypeCompare((PetscObject)eps->st,STSINVERT,&issinv);CHKERRQ(ierr);
-  if (!issinv) SETERRQ(PetscObjectComm((PetscObject)eps),PETSC_ERR_SUP,"Must use STSINVERT spectral transformation");
-
   if (!ctx->lme) { ierr = EPSLyapIIGetLME(eps,&ctx->lme);CHKERRQ(ierr); }
   ierr = LMESetProblemType(ctx->lme,LME_LYAPUNOV);CHKERRQ(ierr);
   ierr = LMESetErrorIfNotConverged(ctx->lme,PETSC_TRUE);CHKERRQ(ierr);
@@ -92,6 +86,7 @@ PetscErrorCode EPSSetUp_LyapII(EPS eps)
   ierr = DSAllocate(eps->ds,eps->ncv);CHKERRQ(ierr);
 
   ierr = EPSAllocateSolution(eps,0);CHKERRQ(ierr);
+  ierr = BVGetRandomContext(eps->V,&rand);CHKERRQ(ierr);  /* make sure the random context is available when duplicating */
   ierr = EPSSetWorkVecs(eps,3);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -126,8 +121,8 @@ static PetscErrorCode MatMult_EigOperator(Mat M,Vec x,Vec y)
   EPS_EIG_MATSHELL  *matctx;
 #if !defined(PETSC_USE_COMPLEX)
   PetscInt          n;
-  PetscScalar       *S,*Y,*C,zero=0.0,done=1.0,dtwo=2.0;
-  const PetscScalar *X;
+  PetscScalar       *Y,*C,zero=0.0,done=1.0,dtwo=2.0;
+  const PetscScalar *S,*X;
   PetscBLASInt      n_;
 #endif
 
@@ -140,7 +135,7 @@ static PetscErrorCode MatMult_EigOperator(Mat M,Vec x,Vec y)
 #else
   ierr = VecGetArrayRead(x,&X);CHKERRQ(ierr);
   ierr = VecGetArray(y,&Y);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(matctx->S,&S);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(matctx->S,&S);CHKERRQ(ierr);
 
   n = matctx->n;
   ierr = PetscCalloc1(n*n,&C);CHKERRQ(ierr);
@@ -151,12 +146,12 @@ static PetscErrorCode MatMult_EigOperator(Mat M,Vec x,Vec y)
   PetscStackCallBLAS("BLASgemm",BLASgemm_("N","T",&n_,&n_,&n_,&done,Y,&n_,S,&n_,&zero,C,&n_));
 
   /* Solve S*Y + Y*S' = -C */
-  ierr = LMEDenseLyapunov(matctx->lme,n,S,n,C,n,Y,n);CHKERRQ(ierr);
+  ierr = LMEDenseLyapunov(matctx->lme,n,(PetscScalar*)S,n,C,n,Y,n);CHKERRQ(ierr);
 
   ierr = PetscFree(C);CHKERRQ(ierr);
   ierr = VecRestoreArrayRead(x,&X);CHKERRQ(ierr);
   ierr = VecRestoreArray(y,&Y);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(matctx->S,&S);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(matctx->S,&S);CHKERRQ(ierr);
 #endif
   PetscFunctionReturn(0);
 }
@@ -184,18 +179,18 @@ static PetscErrorCode MatDestroy_EigOperator(Mat M)
 static PetscErrorCode EV2x2(PetscScalar *M,PetscInt ld,PetscScalar *wr,PetscScalar *wi,PetscScalar *vec)
 {
   PetscErrorCode ierr;
-  PetscScalar    work[10];
   PetscBLASInt   lwork=10,ld_;
 #if !defined(PETSC_HAVE_ESSL)
+  PetscScalar    work[10];
   PetscBLASInt   two=2,info;
-#if defined(PETSC_USE_COMPLEX)
-  PetscReal      rwork[4];
-#endif
 #else
   PetscInt       i;
   PetscBLASInt   idummy,io=1;
   PetscScalar    wri[4];
 #endif
+#if defined(PETSC_HAVE_ESSL) || defined(PETSC_USE_COMPLEX)
+  PetscReal      rwork[6];
+#endif
 
   PetscFunctionBegin;
   ierr = PetscBLASIntCast(ld,&ld_);CHKERRQ(ierr);
@@ -208,7 +203,7 @@ static PetscErrorCode EV2x2(PetscScalar *M,PetscInt ld,PetscScalar *wr,PetscScal
 #endif
   SlepcCheckLapackInfo("geev",info);
 #else /* defined(PETSC_HAVE_ESSL) */
-  PetscStackCallBLAS("LAPACKgeev",LAPACKgeev_(&io,M,&ld_,wri,vec,ld_,&idummy,&ld_,work,&lwork));
+  PetscStackCallBLAS("LAPACKgeev",LAPACKgeev_(&io,M,&ld_,wri,vec,&ld_,&idummy,&ld_,rwork,&lwork));
 #if !defined(PETSC_USE_COMPLEX)
   for (i=0;i<2;i++) {
     wr[i] = wri[2*i];
@@ -252,7 +247,7 @@ static PetscErrorCode LyapIIBuildRHS(Mat S,PetscInt rk,Mat U,BV V,Vec *work)
     } else {
       ierr = VecResetArray(v);CHKERRQ(ierr);
     }
-    ierr = VecScale(u,PetscSqrtReal(2.0));CHKERRQ(ierr);
+    ierr = VecScale(u,PETSC_SQRT2);CHKERRQ(ierr);
     ierr = VecGetArray(u,&uu);CHKERRQ(ierr);
     ierr = PetscMemcpy(array,uu,nloc*sizeof(PetscScalar));CHKERRQ(ierr);
     ierr = VecRestoreArray(u,&uu);CHKERRQ(ierr);
@@ -268,14 +263,15 @@ static PetscErrorCode LyapIIBuildRHS(Mat S,PetscInt rk,Mat U,BV V,Vec *work)
  */
 static PetscErrorCode LyapIIBuildEigenMat(LME lme,Mat S,Mat *Op,Vec *v0)
 {
-  PetscErrorCode   ierr;
-  PetscInt         n,m;
-  PetscBool        create=PETSC_FALSE;
-  EPS_EIG_MATSHELL *matctx;
+  PetscErrorCode    ierr;
+  PetscInt          n,m;
+  PetscBool         create=PETSC_FALSE;
+  EPS_EIG_MATSHELL  *matctx;
 #if defined(PETSC_USE_COMPLEX)
-  PetscScalar      theta,*aa,*bb,*ss;
-  PetscInt         i,j,f,c,off,ld;
-  IS               perm;
+  PetscScalar       theta,*aa,*bb;
+  const PetscScalar *ss;
+  PetscInt          i,j,f,c,off,ld;
+  IS                perm;
 #endif
 
   PetscFunctionBegin;
@@ -307,7 +303,7 @@ static PetscErrorCode LyapIIBuildEigenMat(LME lme,Mat S,Mat *Op,Vec *v0)
 #if defined(PETSC_USE_COMPLEX)
   ierr = MatDenseGetArray(matctx->A,&aa);CHKERRQ(ierr);
   ierr = MatDenseGetArray(matctx->B,&bb);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(S,&ss);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayRead(S,&ss);CHKERRQ(ierr);
   ld = n*n;
   for (f=0;f<n;f++) {
     off = f*n+f*n*ld;
@@ -321,7 +317,7 @@ static PetscErrorCode LyapIIBuildEigenMat(LME lme,Mat S,Mat *Op,Vec *v0)
   }
   ierr = MatDenseRestoreArray(matctx->A,&aa);CHKERRQ(ierr);
   ierr = MatDenseRestoreArray(matctx->B,&bb);CHKERRQ(ierr);
-  ierr = MatDenseRestoreArray(S,&ss);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayRead(S,&ss);CHKERRQ(ierr);
   ierr = ISCreateStride(PETSC_COMM_SELF,n*n,0,1,&perm);CHKERRQ(ierr);
   ierr = MatDestroy(&matctx->F);CHKERRQ(ierr);
   ierr = MatDuplicate(matctx->A,MAT_COPY_VALUES,&matctx->F);CHKERRQ(ierr);
@@ -601,7 +597,7 @@ static PetscErrorCode EPSLyapIISetRanks_LyapII(EPS eps,PetscInt rkc,PetscInt rkl
 /*@
    EPSLyapIISetRanks - Set the ranks used in the solution of the Lyapunov equation.
 
-   Collective on EPS
+   Logically Collective on EPS
 
    Input Parameters:
 +  eps - the eigenproblem solver context
@@ -809,7 +805,7 @@ PetscErrorCode EPSSetDefaultST_LyapII(EPS eps)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode EPSCreate_LyapII(EPS eps)
+SLEPC_EXTERN PetscErrorCode EPSCreate_LyapII(EPS eps)
 {
   EPS_LYAPII     *ctx;
   PetscErrorCode ierr;
@@ -822,6 +818,7 @@ PETSC_EXTERN PetscErrorCode EPSCreate_LyapII(EPS eps)
 
   eps->ops->solve          = EPSSolve_LyapII;
   eps->ops->setup          = EPSSetUp_LyapII;
+  eps->ops->setupsort      = EPSSetUpSort_Default;
   eps->ops->setfromoptions = EPSSetFromOptions_LyapII;
   eps->ops->reset          = EPSReset_LyapII;
   eps->ops->destroy        = EPSDestroy_LyapII;
