@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -35,33 +35,23 @@ PetscErrorCode PEPSetUp_QArnoldi(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_QARNOLDI   *ctx = (PEP_QARNOLDI*)pep->data;
-  PetscBool      shift,sinv,flg;
+  PetscBool      flg;
 
   PetscFunctionBegin;
-  pep->lineariz = PETSC_TRUE;
+  PEPCheckQuadratic(pep);
+  PEPCheckShiftSinvert(pep);
   ierr = PEPSetDimensions_Default(pep,pep->nev,&pep->ncv,&pep->mpd);CHKERRQ(ierr);
   if (!ctx->lock && pep->mpd<pep->ncv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Should not use mpd parameter in non-locking variant");
-  if (!pep->max_it) pep->max_it = PetscMax(100,4*pep->n/pep->ncv);
+  if (pep->max_it==PETSC_DEFAULT) pep->max_it = PetscMax(100,4*pep->n/pep->ncv);
+  if (!pep->which) { ierr = PEPSetWhichEigenpairs_Default(pep);CHKERRQ(ierr); }
+  if (pep->which==PEP_ALL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"This solver does not support computing all eigenvalues");
 
-  ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSHIFT,&shift);CHKERRQ(ierr);
-  ierr = PetscObjectTypeCompare((PetscObject)pep->st,STSINVERT,&sinv);CHKERRQ(ierr);
-  if (!shift && !sinv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Only STSHIFT and STSINVERT spectral transformations can be used");
-  if (!pep->which) {
-    if (sinv) pep->which = PEP_TARGET_MAGNITUDE;
-    else pep->which = PEP_LARGEST_MAGNITUDE;
-  }
-  if (pep->which==PEP_ALL) SETERRQ(PetscObjectComm((PetscObject)pep),1,"Wrong value of pep->which");
-
-  if (pep->nmat!=3) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver only available for quadratic problems");
-  if (pep->basis!=PEP_BASIS_MONOMIAL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver not implemented for non-monomial bases");
   ierr = STGetTransform(pep->st,&flg);CHKERRQ(ierr);
   if (!flg) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver requires the ST transformation flag set, see STSetTransform()");
 
   /* set default extraction */
-  if (!pep->extract) {
-    pep->extract = PEP_EXTRACT_NONE;
-  }
-  if (pep->extract!=PEP_EXTRACT_NONE) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Solver does not support requested extraction");
+  if (!pep->extract) pep->extract = PEP_EXTRACT_NONE;
+  PEPCheckUnsupported(pep,PEP_FEATURE_NONMONOMIAL | PEP_FEATURE_EXTRACT);
 
   if (!ctx->keep) ctx->keep = 0.5;
 
@@ -90,11 +80,11 @@ PetscErrorCode PEPExtractVectors_QArnoldi(PEP pep)
 
   /* update vectors V = V*X */
   ierr = MatCreateSeqDense(PETSC_COMM_SELF,k,k,NULL,&X0);CHKERRQ(ierr);
-  ierr = MatDenseGetArray(X0,&pX0);CHKERRQ(ierr);
+  ierr = MatDenseGetArrayWrite(X0,&pX0);CHKERRQ(ierr);
   for (i=0;i<k;i++) {
     ierr = PetscArraycpy(pX0+i*k,X+i*ldds,k);CHKERRQ(ierr);
   }
-  ierr = MatDenseRestoreArray(X0,&pX0);CHKERRQ(ierr);
+  ierr = MatDenseRestoreArrayWrite(X0,&pX0);CHKERRQ(ierr);
   ierr = BVMultInPlace(pep->V,X0,0,k);CHKERRQ(ierr);
   ierr = MatDestroy(&X0);CHKERRQ(ierr);
   ierr = DSRestoreArray(pep->ds,DS_MAT_X,&X);CHKERRQ(ierr);
@@ -223,7 +213,7 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
 {
   PetscErrorCode ierr;
   PEP_QARNOLDI   *ctx = (PEP_QARNOLDI*)pep->data;
-  PetscInt       j,k,l,lwork,nv,ld,newn,nconv;
+  PetscInt       j,k,l,lwork,nv,ld,nconv;
   Vec            v=pep->work[0],w=pep->work[1];
   Mat            Q;
   PetscScalar    *S,*work;
@@ -288,8 +278,12 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
 
     /* Update l */
     if (pep->reason != PEP_CONVERGED_ITERATING || breakdown) l = 0;
-    else l = PetscMax(1,(PetscInt)((nv-k)*ctx->keep));
+    else {
+      l = PetscMax(1,(PetscInt)((nv-k)*ctx->keep));
+      ierr = DSGetTruncateSize(pep->ds,k,nv,&l);CHKERRQ(ierr);
+    }
     if (!ctx->lock && l>0) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
+    if (l) { ierr = PetscInfo1(pep,"Preparing to restart keeping l=%D vectors\n",l);CHKERRQ(ierr); }
 
     if (pep->reason == PEP_CONVERGED_ITERATING) {
       if (breakdown) {
@@ -298,9 +292,7 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
         pep->reason = PEP_DIVERGED_BREAKDOWN;
       } else {
         /* Prepare the Rayleigh quotient for restart */
-        ierr = DSTruncate(pep->ds,k+l);CHKERRQ(ierr);
-        ierr = DSGetDimensions(pep->ds,&newn,NULL,NULL,NULL,NULL);CHKERRQ(ierr);
-        l = newn-k;
+        ierr = DSTruncate(pep->ds,k+l,PETSC_FALSE);CHKERRQ(ierr);
       }
     }
     /* Update the corresponding vectors V(:,idx) = V*Q(:,idx) */
@@ -320,10 +312,7 @@ PetscErrorCode PEPSolve_QArnoldi(PEP pep)
   ierr = STScaleShift(pep->st,sinv?1.0/pep->sfactor:pep->sfactor);CHKERRQ(ierr);
   ierr = RGPopScale(pep->rg);CHKERRQ(ierr);
 
-  /* truncate Schur decomposition and change the state to raw so that
-     DSVectors() computes eigenvectors from scratch */
-  ierr = DSSetDimensions(pep->ds,pep->nconv,0,0,0);CHKERRQ(ierr);
-  ierr = DSSetState(pep->ds,DS_STATE_RAW);CHKERRQ(ierr);
+  ierr = DSTruncate(pep->ds,pep->nconv,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscFree(work);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
@@ -543,7 +532,9 @@ SLEPC_EXTERN PetscErrorCode PEPCreate_QArnoldi(PEP pep)
   PetscFunctionBegin;
   ierr = PetscNewLog(pep,&ctx);CHKERRQ(ierr);
   pep->data = (void*)ctx;
-  ctx->lock = PETSC_TRUE;
+
+  pep->lineariz = PETSC_TRUE;
+  ctx->lock     = PETSC_TRUE;
 
   pep->ops->solve          = PEPSolve_QArnoldi;
   pep->ops->setup          = PEPSetUp_QArnoldi;
