@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -20,7 +20,7 @@
 
        [1] C. Campos and J.E. Roman, "Inertia-based spectrum slicing
            for symmetric quadratic eigenvalue problems", Numer. Linear
-           Algebra Appl. (in press), 2020.
+           Algebra Appl. 27(4):e2293, 2020.
 */
 
 #include <slepc/private/pepimpl.h>         /*I "slepcpep.h" I*/
@@ -33,9 +33,9 @@ static const char citation[] =
   "   author = \"C. Campos and J. E. Roman\",\n"
   "   title = \"Inertia-based spectrum slicing for symmetric quadratic eigenvalue problems\",\n"
   "   journal = \"Numer. Linear Algebra Appl.\",\n"
-  "   volume = \"IP\",\n"
-  "   number = \"x\",\n"
-  "   pages = \"xx--xx\",\n"
+  "   volume = \"27\",\n"
+  "   number = \"4\",\n"
+  "   pages = \"e2293\",\n"
   "   year = \"2020,\"\n"
   "   doi = \"https://doi.org/10.1002/nla.2293\"\n"
   "}\n";
@@ -502,18 +502,25 @@ PetscErrorCode PEPSetUp_STOAR_QSlice(PEP pep)
   PEP_SR         sr;
   PetscInt       ld,i,zeros=0;
   SlepcSC        sc;
-  PetscBool      issinv;
   PetscReal      r;
 
   PetscFunctionBegin;
+  PEPCheckSinvertCayley(pep);
+  if (pep->inta==pep->intb) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"This solver does not support computing all eigenvalues unless you provide a computational interval with PEPSetInterval()");
   if (pep->intb >= PETSC_MAX_REAL && pep->inta <= PETSC_MIN_REAL) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"The defined computational interval should have at least one of their sides bounded");
-  ierr = PetscObjectTypeCompareAny((PetscObject)pep->st,&issinv,STSINVERT,STCAYLEY,"");CHKERRQ(ierr);
-  if (!issinv) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_SUP,"Shift-and-invert or Cayley ST is needed for spectrum slicing");
-  if (pep->tol==PETSC_DEFAULT) pep->tol = SLEPC_DEFAULT_TOL*1e-2;  /* use tighter tolerance */
+  PEPCheckUnsupportedCondition(pep,PEP_FEATURE_STOPPING,PETSC_TRUE," (with spectrum slicing)");
+  if (pep->tol==PETSC_DEFAULT) {
+#if defined(PETSC_USE_REAL_SINGLE)
+    pep->tol = SLEPC_DEFAULT_TOL;
+#else
+    /* use tighter tolerance */
+    pep->tol = SLEPC_DEFAULT_TOL*1e-2;
+#endif
+  }
   if (ctx->nev==1) ctx->nev = PetscMin(20,pep->n);  /* nev not set, use default value */
   if (pep->n>10 && ctx->nev<10) SETERRQ(PetscObjectComm((PetscObject)pep),PETSC_ERR_ARG_WRONG,"nev cannot be less than 10 in spectrum slicing runs");
   pep->ops->backtransform = PEPBackTransform_Skip;
-  if (!pep->max_it) pep->max_it = 100;
+  if (pep->max_it==PETSC_DEFAULT) pep->max_it = 100;
 
   /* create spectrum slicing context and initialize it */
   ierr = PEPQSliceResetSR(pep);CHKERRQ(ierr);
@@ -585,6 +592,7 @@ PetscErrorCode PEPSetUp_STOAR_QSlice(PEP pep)
     ld   = ctx->ncv+2;
     ierr = DSSetType(pep->ds,DSGHIEP);CHKERRQ(ierr);
     ierr = DSSetCompact(pep->ds,PETSC_TRUE);CHKERRQ(ierr);
+    ierr = DSSetExtraRow(pep->ds,PETSC_TRUE);CHKERRQ(ierr);
     ierr = DSAllocate(pep->ds,ld);CHKERRQ(ierr);
     ierr = DSGetSlepcSC(pep->ds,&sc);CHKERRQ(ierr);
     sc->rg            = NULL;
@@ -592,7 +600,7 @@ PetscErrorCode PEPSetUp_STOAR_QSlice(PEP pep)
     sc->comparisonctx = NULL;
     sc->map           = NULL;
     sc->mapobj        = NULL;
-  }
+  } else {pep->ncv = 0; pep->nev = 0; pep->mpd = 0;}
   PetscFunctionReturn(0);
 }
 
@@ -1100,8 +1108,8 @@ static PetscErrorCode PEPSTOAR_QSlice(PEP pep,Mat B)
   PEP_STOAR      *ctx = (PEP_STOAR*)pep->data;
   PetscInt       j,k,l,nv=0,ld,ldds,t,nq=0,idx;
   PetscInt       nconv=0,deg=pep->nmat-1,count0=0,count1=0;
-  PetscScalar    *Q,*om,sigma,*back,*S,*pQ;
-  PetscReal      beta,norm=1.0,*omega,*a,*b,*r,eta,lambda;
+  PetscScalar    *om,sigma,*back,*S,*pQ;
+  PetscReal      beta,norm=1.0,*omega,*a,*b,eta,lambda;
   PetscBool      breakdown,symmlost=PETSC_FALSE,sinv,falselock=PETSC_TRUE;
   Mat            MS,MQ;
   Vec            v,vomega;
@@ -1253,6 +1261,7 @@ static PetscErrorCode PEPSTOAR_QSlice(PEP pep,Mat B)
     /* Solve projected problem */
     ierr = DSSolve(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
     ierr = DSSort(pep->ds,pep->eigr,pep->eigi,NULL,NULL,NULL);CHKERRQ(ierr);
+    ierr = DSUpdateExtraRow(pep->ds);CHKERRQ(ierr);
     ierr = DSSynchronize(pep->ds,pep->eigr,pep->eigi);CHKERRQ(ierr);
 
     /* Check convergence */
@@ -1275,29 +1284,15 @@ static PetscErrorCode PEPSTOAR_QSlice(PEP pep,Mat B)
     else {
       l = PetscMax(1,(PetscInt)((nv-k)/2));
       l = PetscMin(l,t);
+      ierr = DSGetTruncateSize(pep->ds,k,t,&l);CHKERRQ(ierr);
       if (!breakdown) {
-        ierr = DSGetArrayReal(pep->ds,DS_MAT_T,&a);CHKERRQ(ierr);
-        if (*(a+ldds+k+l-1)!=0) {
-          if (k+l<nv-1) l = l+1;
-          else l = l-1;
-        }
         /* Prepare the Rayleigh quotient for restart */
-        ierr = DSGetArray(pep->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-        ierr = DSGetArrayReal(pep->ds,DS_MAT_D,&omega);CHKERRQ(ierr);
-        r = a + 2*ldds;
-        for (j=k;j<k+l;j++) {
-          r[j] = PetscRealPart(Q[nv-1+j*ldds]*beta);
-        }
-        b = a+ldds;
-        b[k+l-1] = r[k+l-1];
-        omega[k+l] = omega[nv];
-        ierr = DSRestoreArray(pep->ds,DS_MAT_Q,&Q);CHKERRQ(ierr);
-        ierr = DSRestoreArrayReal(pep->ds,DS_MAT_T,&a);CHKERRQ(ierr);
-        ierr = DSRestoreArrayReal(pep->ds,DS_MAT_D,&omega);CHKERRQ(ierr);
+        ierr = DSTruncate(pep->ds,k+l,PETSC_FALSE);CHKERRQ(ierr);
       }
     }
     nconv = k;
     if (!ctx->lock && pep->reason == PEP_CONVERGED_ITERATING && !breakdown) { l += k; k = 0; } /* non-locking variant: reset no. of converged pairs */
+    if (l) { ierr = PetscInfo1(pep,"Preparing to restart keeping l=%D vectors\n",l);CHKERRQ(ierr); }
 
     /* Update S */
     ierr = DSGetMat(pep->ds,DS_MAT_Q,&MQ);CHKERRQ(ierr);
@@ -1357,10 +1352,7 @@ static PetscErrorCode PEPSTOAR_QSlice(PEP pep,Mat B)
     if (++sr->symmlost>10) SETERRQ1(PetscObjectComm((PetscObject)pep),1,"Symmetry lost at sigma=%g",(double)sr->sPres->value);
   } else sr->symmlost = 0;
 
-  /* truncate Schur decomposition and change the state to raw so that
-     DSVectors() computes eigenvectors from scratch */
-  ierr = DSSetDimensions(pep->ds,pep->nconv,0,0,0);CHKERRQ(ierr);
-  ierr = DSSetState(pep->ds,DS_STATE_RAW);CHKERRQ(ierr);
+  ierr = DSTruncate(pep->ds,pep->nconv,PETSC_TRUE);CHKERRQ(ierr);
   ierr = PetscFree(back);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
