@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -36,7 +36,7 @@ int main(int argc,char **argv)
   Mat            M,C,K,A[3];      /* problem matrices */
   PEP            pep;             /* polynomial eigenproblem solver context */
   PetscInt       n=5,Istart,Iend,i;
-  PetscReal      mu=1,tau=10,kappa=5;
+  PetscReal      mu=1.0,tau=10.0,kappa=5.0;
   PetscBool      terse;
   PetscErrorCode ierr;
 
@@ -150,6 +150,10 @@ int main(int argc,char **argv)
       test:
          suffix: 1_qarnoldi
          args: -pep_type qarnoldi -pep_conv_rel
+      test:
+         suffix: 1_cuda
+         args: -mat_type aijcusparse
+         requires: cuda
 
    test:
       suffix: 2
@@ -159,6 +163,7 @@ int main(int argc,char **argv)
    test:
       suffix: 3
       args: -n 300 -pep_hermitian -pep_interval -10.1,-9.5 -pep_type stoar -st_type sinvert -st_pc_type cholesky -terse
+      filter: sed -e "s/52565/52566/" | sed -e "s/90758/90759/"
 
    test:
       suffix: 4
