@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -33,8 +33,8 @@ static PetscErrorCode MatMult_Cross(Mat B,Vec x,Vec y)
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&svd);CHKERRQ(ierr);
   cross = (SVD_CROSS*)svd->data;
-  ierr = SVDMatMult(svd,PETSC_FALSE,x,cross->w);CHKERRQ(ierr);
-  ierr = SVDMatMult(svd,PETSC_TRUE,cross->w,y);CHKERRQ(ierr);
+  ierr = MatMult(svd->A,x,cross->w);CHKERRQ(ierr);
+  ierr = MatMult(svd->AT,cross->w,y);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -46,10 +46,10 @@ static PetscErrorCode MatCreateVecs_Cross(Mat B,Vec *right,Vec *left)
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&svd);CHKERRQ(ierr);
   if (right) {
-    ierr = SVDMatCreateVecs(svd,right,NULL);CHKERRQ(ierr);
+    ierr = MatCreateVecs(svd->A,right,NULL);CHKERRQ(ierr);
     if (left) { ierr = VecDuplicate(*right,left);CHKERRQ(ierr); }
   } else {
-    ierr = SVDMatCreateVecs(svd,left,NULL);CHKERRQ(ierr);
+    ierr = MatCreateVecs(svd->A,left,NULL);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -71,10 +71,10 @@ static PetscErrorCode MatGetDiagonal_Cross(Mat B,Vec d)
   if (!cross->diag) {
     /* compute diagonal from rows and store in cross->diag */
     ierr = VecDuplicate(d,&cross->diag);CHKERRQ(ierr);
-    ierr = SVDMatGetSize(svd,NULL,&N);CHKERRQ(ierr);
-    ierr = SVDMatGetLocalSize(svd,NULL,&n);CHKERRQ(ierr);
+    ierr = MatGetSize(svd->A,NULL,&N);CHKERRQ(ierr);
+    ierr = MatGetLocalSize(svd->A,NULL,&n);CHKERRQ(ierr);
     ierr = PetscCalloc2(N,&work1,N,&work2);CHKERRQ(ierr);
-    if (svd->AT) {
+    if (svd->swapped) {
       ierr = MatGetOwnershipRange(svd->AT,&start,&end);CHKERRQ(ierr);
       for (i=start;i<end;i++) {
         ierr = MatGetRow(svd->AT,i,&ncols,NULL,&vals);CHKERRQ(ierr);
@@ -92,7 +92,7 @@ static PetscErrorCode MatGetDiagonal_Cross(Mat B,Vec d)
       }
     }
     ierr = PetscMPIIntCast(N,&len);CHKERRQ(ierr);
-    ierr = MPI_Allreduce(work1,work2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)svd));CHKERRQ(ierr);
+    ierr = MPIU_Allreduce(work1,work2,len,MPIU_SCALAR,MPIU_SUM,PetscObjectComm((PetscObject)svd));CHKERRQ(ierr);
     ierr = VecGetOwnershipRange(cross->diag,&start,&end);CHKERRQ(ierr);
     ierr = VecGetArray(cross->diag,&diag);CHKERRQ(ierr);
     for (i=start;i<end;i++) diag[i-start] = work2[i];
@@ -111,16 +111,17 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
   PetscBool      trackall;
 
   PetscFunctionBegin;
+  SVDCheckStandard(svd);
   if (!cross->mat) {
     if (cross->explicitmatrix) {
-      if (svd->A && svd->AT) {  /* explicit transpose */
+      if (svd->expltrans) {  /* explicit transpose */
         ierr = MatProductCreate(svd->AT,svd->A,NULL,&cross->mat);CHKERRQ(ierr);
         ierr = MatProductSetType(cross->mat,MATPRODUCT_AB);CHKERRQ(ierr);
       } else {  /* implicit transpose */
 #if defined(PETSC_USE_COMPLEX)
         SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Must use explicit transpose with complex scalars");
 #else
-        if (svd->A) {
+        if (!svd->swapped) {
           ierr = MatProductCreate(svd->A,svd->A,NULL,&cross->mat);CHKERRQ(ierr);
           ierr = MatProductSetType(cross->mat,MATPRODUCT_AtB);CHKERRQ(ierr);
         } else {
@@ -133,12 +134,12 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
       ierr = MatProductSymbolic(cross->mat);CHKERRQ(ierr);
       ierr = MatProductNumeric(cross->mat);CHKERRQ(ierr);
     } else {
-      ierr = SVDMatGetLocalSize(svd,NULL,&n);CHKERRQ(ierr);
+      ierr = MatGetLocalSize(svd->A,NULL,&n);CHKERRQ(ierr);
       ierr = MatCreateShell(PetscObjectComm((PetscObject)svd),n,n,PETSC_DETERMINE,PETSC_DETERMINE,svd,&cross->mat);CHKERRQ(ierr);
       ierr = MatShellSetOperation(cross->mat,MATOP_MULT,(void(*)(void))MatMult_Cross);CHKERRQ(ierr);
       ierr = MatShellSetOperation(cross->mat,MATOP_CREATE_VECS,(void(*)(void))MatCreateVecs_Cross);CHKERRQ(ierr);
       ierr = MatShellSetOperation(cross->mat,MATOP_GET_DIAGONAL,(void(*)(void))MatGetDiagonal_Cross);CHKERRQ(ierr);
-      ierr = SVDMatCreateVecs(svd,NULL,&cross->w);CHKERRQ(ierr);
+      ierr = MatCreateVecs(svd->A,NULL,&cross->w);CHKERRQ(ierr);
       ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cross->w);CHKERRQ(ierr);
     }
     ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cross->mat);CHKERRQ(ierr);
@@ -149,18 +150,20 @@ PetscErrorCode SVDSetUp_Cross(SVD svd)
   ierr = EPSSetProblemType(cross->eps,EPS_HEP);CHKERRQ(ierr);
   if (!cross->usereps) {
     ierr = EPSSetWhichEigenpairs(cross->eps,svd->which==SVD_LARGEST?EPS_LARGEST_REAL:EPS_SMALLEST_REAL);CHKERRQ(ierr);
-    ierr = EPSSetDimensions(cross->eps,svd->nsv,svd->ncv?svd->ncv:PETSC_DEFAULT,svd->mpd?svd->mpd:PETSC_DEFAULT);CHKERRQ(ierr);
-    ierr = EPSSetTolerances(cross->eps,svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/10.0:svd->tol,svd->max_it?svd->max_it:PETSC_DEFAULT);CHKERRQ(ierr);
+    ierr = EPSSetDimensions(cross->eps,svd->nsv,svd->ncv,svd->mpd);CHKERRQ(ierr);
+    ierr = EPSSetTolerances(cross->eps,svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/10.0:svd->tol,svd->max_it);CHKERRQ(ierr);
     switch (svd->conv) {
     case SVD_CONV_ABS:
       ierr = EPSSetConvergenceTest(cross->eps,EPS_CONV_ABS);CHKERRQ(ierr);break;
     case SVD_CONV_REL:
       ierr = EPSSetConvergenceTest(cross->eps,EPS_CONV_REL);CHKERRQ(ierr);break;
+    case SVD_CONV_MAXIT:
+      SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Maxit convergence test not supported in this solver");
     case SVD_CONV_USER:
       SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"User-defined convergence test not supported in this solver");
     }
   }
-  if (svd->stop!=SVD_STOP_BASIC) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"User-defined stopping test not supported in this solver");
+  SVDCheckUnsupported(svd,SVD_FEATURE_STOPPING);
   /* Transfer the trackall option from svd to eps */
   ierr = SVDGetTrackAll(svd,&trackall);CHKERRQ(ierr);
   ierr = EPSSetTrackAll(cross->eps,trackall);CHKERRQ(ierr);
@@ -186,7 +189,6 @@ PetscErrorCode SVDSolve_Cross(SVD svd)
   PetscInt       i;
   PetscScalar    lambda;
   PetscReal      sigma;
-  Vec            v;
 
   PetscFunctionBegin;
   ierr = EPSSolve(cross->eps);CHKERRQ(ierr);
@@ -194,9 +196,7 @@ PetscErrorCode SVDSolve_Cross(SVD svd)
   ierr = EPSGetIterationNumber(cross->eps,&svd->its);CHKERRQ(ierr);
   ierr = EPSGetConvergedReason(cross->eps,(EPSConvergedReason*)&svd->reason);CHKERRQ(ierr);
   for (i=0;i<svd->nconv;i++) {
-    ierr = BVGetColumn(svd->V,i,&v);CHKERRQ(ierr);
-    ierr = EPSGetEigenpair(cross->eps,i,&lambda,NULL,v,NULL);CHKERRQ(ierr);
-    ierr = BVRestoreColumn(svd->V,i,&v);CHKERRQ(ierr);
+    ierr = EPSGetEigenvalue(cross->eps,i,&lambda,NULL);CHKERRQ(ierr);
     sigma = PetscRealPart(lambda);
     if (sigma<-10*PETSC_MACHINE_EPSILON) SETERRQ1(PetscObjectComm((PetscObject)svd),1,"Negative eigenvalue computed by EPS: %g",sigma);
     if (sigma<0.0) {
@@ -208,6 +208,23 @@ PetscErrorCode SVDSolve_Cross(SVD svd)
   PetscFunctionReturn(0);
 }
 
+PetscErrorCode SVDComputeVectors_Cross(SVD svd)
+{
+  PetscErrorCode ierr;
+  SVD_CROSS      *cross = (SVD_CROSS*)svd->data;
+  PetscInt       i;
+  Vec            v;
+
+  PetscFunctionBegin;
+  for (i=0;i<svd->nconv;i++) {
+    ierr = BVGetColumn(svd->V,i,&v);CHKERRQ(ierr);
+    ierr = EPSGetEigenvector(cross->eps,i,v,NULL);CHKERRQ(ierr);
+    ierr = BVRestoreColumn(svd->V,i,&v);CHKERRQ(ierr);
+  }
+  ierr = SVDComputeVectors_Left(svd);CHKERRQ(ierr);
+  PetscFunctionReturn(0);
+}
+
 static PetscErrorCode EPSMonitor_Cross(EPS eps,PetscInt its,PetscInt nconv,PetscScalar *eigr,PetscScalar *eigi,PetscReal *errest,PetscInt nest,void *ctx)
 {
   PetscInt       i;
@@ -475,6 +492,7 @@ SLEPC_EXTERN PetscErrorCode SVDCreate_Cross(SVD svd)
   svd->ops->destroy        = SVDDestroy_Cross;
   svd->ops->reset          = SVDReset_Cross;
   svd->ops->view           = SVDView_Cross;
+  svd->ops->computevectors = SVDComputeVectors_Cross;
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDCrossSetEPS_C",SVDCrossSetEPS_Cross);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDCrossGetEPS_C",SVDCrossGetEPS_Cross);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDCrossSetExplicitMatrix_C",SVDCrossSetExplicitMatrix_Cross);CHKERRQ(ierr);
