@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -29,15 +29,15 @@ static PetscErrorCode MatMult_Cyclic(Mat B,Vec x,Vec y)
   PetscFunctionBegin;
   ierr = MatShellGetContext(B,(void**)&svd);CHKERRQ(ierr);
   cyclic = (SVD_CYCLIC*)svd->data;
-  ierr = SVDMatGetLocalSize(svd,&m,NULL);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(svd->A,&m,NULL);CHKERRQ(ierr);
   ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
   ierr = VecGetArray(y,&py);CHKERRQ(ierr);
   ierr = VecPlaceArray(cyclic->x1,px);CHKERRQ(ierr);
   ierr = VecPlaceArray(cyclic->x2,px+m);CHKERRQ(ierr);
   ierr = VecPlaceArray(cyclic->y1,py);CHKERRQ(ierr);
   ierr = VecPlaceArray(cyclic->y2,py+m);CHKERRQ(ierr);
-  ierr = SVDMatMult(svd,PETSC_FALSE,cyclic->x2,cyclic->y1);CHKERRQ(ierr);
-  ierr = SVDMatMult(svd,PETSC_TRUE,cyclic->x1,cyclic->y2);CHKERRQ(ierr);
+  ierr = MatMult(svd->A,cyclic->x2,cyclic->y1);CHKERRQ(ierr);
+  ierr = MatMult(svd->AT,cyclic->x1,cyclic->y2);CHKERRQ(ierr);
   ierr = VecResetArray(cyclic->x1);CHKERRQ(ierr);
   ierr = VecResetArray(cyclic->x2);CHKERRQ(ierr);
   ierr = VecResetArray(cyclic->y1);CHKERRQ(ierr);
@@ -72,11 +72,12 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
 #endif
 
   PetscFunctionBegin;
-  ierr = SVDMatGetSize(svd,&M,&N);CHKERRQ(ierr);
-  ierr = SVDMatGetLocalSize(svd,&m,&n);CHKERRQ(ierr);
+  SVDCheckStandard(svd);
+  ierr = MatGetSize(svd->A,&M,&N);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(svd->A,&m,&n);CHKERRQ(ierr);
   if (!cyclic->mat) {
     if (cyclic->explicitmatrix) {
-      if (!svd->A || !svd->AT) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Cannot use explicit cyclic matrix with implicit transpose");
+      if (!svd->expltrans) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Cannot use explicit cyclic matrix with implicit transpose");
       ierr = MatCreate(PetscObjectComm((PetscObject)svd),&Zm);CHKERRQ(ierr);
       ierr = MatSetSizes(Zm,m,m,M,M);CHKERRQ(ierr);
       ierr = MatSetFromOptions(Zm);CHKERRQ(ierr);
@@ -102,8 +103,8 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
       ierr = MatDestroy(&Zm);CHKERRQ(ierr);
       ierr = MatDestroy(&Zn);CHKERRQ(ierr);
     } else {
-      ierr = SVDMatCreateVecsEmpty(svd,&cyclic->x2,&cyclic->x1);CHKERRQ(ierr);
-      ierr = SVDMatCreateVecsEmpty(svd,&cyclic->y2,&cyclic->y1);CHKERRQ(ierr);
+      ierr = MatCreateVecsEmpty(svd->A,&cyclic->x2,&cyclic->x1);CHKERRQ(ierr);
+      ierr = MatCreateVecsEmpty(svd->A,&cyclic->y2,&cyclic->y1);CHKERRQ(ierr);
       ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cyclic->x1);CHKERRQ(ierr);
       ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cyclic->x2);CHKERRQ(ierr);
       ierr = PetscLogObjectParent((PetscObject)svd,(PetscObject)cyclic->y1);CHKERRQ(ierr);
@@ -113,7 +114,7 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
 #if defined(PETSC_HAVE_CUDA)
       ierr = PetscObjectTypeCompareAny((PetscObject)(svd->A?svd->A:svd->AT),&cuda,MATSEQAIJCUSPARSE,MATMPIAIJCUSPARSE,"");CHKERRQ(ierr);
       if (cuda) {
-        ierr = MatShellSetOperation(cyclic->mat,MATOP_CREATE_VECS,(void(*)(void))MatCreateVecs_Cyclic_CUDA);CHKERRQ(ierr);
+        ierr = MatShellSetVecType(cyclic->mat,VECCUDA);CHKERRQ(ierr);
         ierr = MatShellSetOperation(cyclic->mat,MATOP_MULT,(void(*)(void))MatMult_Cyclic_CUDA);CHKERRQ(ierr);
       } else
 #endif
@@ -140,18 +141,20 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
       ierr = EPSSetEigenvalueComparison(cyclic->eps,SlepcCompareSmallestPosReal,NULL);CHKERRQ(ierr);
       ierr = EPSSetTarget(cyclic->eps,0.0);CHKERRQ(ierr);
     }
-    ierr = EPSSetDimensions(cyclic->eps,svd->nsv,svd->ncv?svd->ncv:PETSC_DEFAULT,svd->mpd?svd->mpd:PETSC_DEFAULT);CHKERRQ(ierr);
-    ierr = EPSSetTolerances(cyclic->eps,svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/10.0:svd->tol,svd->max_it?svd->max_it:PETSC_DEFAULT);CHKERRQ(ierr);
+    ierr = EPSSetDimensions(cyclic->eps,svd->nsv,svd->ncv,svd->mpd);CHKERRQ(ierr);
+    ierr = EPSSetTolerances(cyclic->eps,svd->tol==PETSC_DEFAULT?SLEPC_DEFAULT_TOL/10.0:svd->tol,svd->max_it);CHKERRQ(ierr);
     switch (svd->conv) {
     case SVD_CONV_ABS:
       ierr = EPSSetConvergenceTest(cyclic->eps,EPS_CONV_ABS);CHKERRQ(ierr);break;
     case SVD_CONV_REL:
       ierr = EPSSetConvergenceTest(cyclic->eps,EPS_CONV_REL);CHKERRQ(ierr);break;
+    case SVD_CONV_MAXIT:
+      SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"Maxit convergence test not supported in this solver");
     case SVD_CONV_USER:
       SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"User-defined convergence test not supported in this solver");
     }
   }
-  if (svd->stop!=SVD_STOP_BASIC) SETERRQ(PetscObjectComm((PetscObject)svd),PETSC_ERR_SUP,"User-defined stopping test not supported in this solver");
+  SVDCheckUnsupported(svd,SVD_FEATURE_STOPPING);
   /* Transfer the trackall option from svd to eps */
   ierr = SVDGetTrackAll(svd,&trackall);CHKERRQ(ierr);
   ierr = EPSSetTrackAll(cyclic->eps,trackall);CHKERRQ(ierr);
@@ -199,43 +202,59 @@ PetscErrorCode SVDSetUp_Cyclic(SVD svd)
 }
 
 PetscErrorCode SVDSolve_Cyclic(SVD svd)
+{
+  PetscErrorCode ierr;
+  SVD_CYCLIC     *cyclic = (SVD_CYCLIC*)svd->data;
+  PetscInt       i,j,nconv;
+  PetscScalar    sigma;
+
+  PetscFunctionBegin;
+  ierr = EPSSolve(cyclic->eps);CHKERRQ(ierr);
+  ierr = EPSGetConverged(cyclic->eps,&nconv);CHKERRQ(ierr);
+  ierr = EPSGetIterationNumber(cyclic->eps,&svd->its);CHKERRQ(ierr);
+  ierr = EPSGetConvergedReason(cyclic->eps,(EPSConvergedReason*)&svd->reason);CHKERRQ(ierr);
+  for (i=0,j=0;i<nconv;i++) {
+    ierr = EPSGetEigenvalue(cyclic->eps,i,&sigma,NULL);CHKERRQ(ierr);
+    if (PetscRealPart(sigma) > 0.0) {
+      svd->sigma[j] = PetscRealPart(sigma);
+      j++;
+    }
+  }
+  svd->nconv = j;
+  PetscFunctionReturn(0);
+}
+
+PetscErrorCode SVDComputeVectors_Cyclic(SVD svd)
 {
   PetscErrorCode    ierr;
   SVD_CYCLIC        *cyclic = (SVD_CYCLIC*)svd->data;
-  PetscInt          i,j,M,N,m,n;
+  PetscInt          i,j,M,N,m,n,nconv;
   PetscScalar       sigma;
   const PetscScalar *px;
   Vec               x,x1,x2;
 
   PetscFunctionBegin;
-  ierr = EPSSolve(cyclic->eps);CHKERRQ(ierr);
-  ierr = EPSGetConverged(cyclic->eps,&svd->nconv);CHKERRQ(ierr);
-  ierr = EPSGetIterationNumber(cyclic->eps,&svd->its);CHKERRQ(ierr);
-  ierr = EPSGetConvergedReason(cyclic->eps,(EPSConvergedReason*)&svd->reason);CHKERRQ(ierr);
-
+  ierr = EPSGetConverged(cyclic->eps,&nconv);CHKERRQ(ierr);
   ierr = MatCreateVecs(cyclic->mat,&x,NULL);CHKERRQ(ierr);
-  ierr = SVDMatGetSize(svd,&M,&N);CHKERRQ(ierr);
-  ierr = SVDMatGetLocalSize(svd,&m,&n);CHKERRQ(ierr);
-  ierr = SVDMatCreateVecsEmpty(svd,&x2,&x1);CHKERRQ(ierr);
-  for (i=0,j=0;i<svd->nconv;i++) {
+  ierr = MatGetSize(svd->A,&M,&N);CHKERRQ(ierr);
+  ierr = MatGetLocalSize(svd->A,&m,&n);CHKERRQ(ierr);
+  ierr = MatCreateVecsEmpty(svd->A,&x2,&x1);CHKERRQ(ierr);
+  for (i=0,j=0;i<nconv;i++) {
     ierr = EPSGetEigenpair(cyclic->eps,i,&sigma,NULL,x,NULL);CHKERRQ(ierr);
     if (PetscRealPart(sigma) > 0.0) {
-      svd->sigma[j] = PetscRealPart(sigma);
       ierr = VecGetArrayRead(x,&px);CHKERRQ(ierr);
       ierr = VecPlaceArray(x1,px);CHKERRQ(ierr);
       ierr = VecPlaceArray(x2,px+m);CHKERRQ(ierr);
       ierr = BVInsertVec(svd->U,j,x1);CHKERRQ(ierr);
-      ierr = BVScaleColumn(svd->U,j,1.0/PetscSqrtReal(2.0));CHKERRQ(ierr);
+      ierr = BVScaleColumn(svd->U,j,1.0/PETSC_SQRT2);CHKERRQ(ierr);
       ierr = BVInsertVec(svd->V,j,x2);CHKERRQ(ierr);
-      ierr = BVScaleColumn(svd->V,j,1.0/PetscSqrtReal(2.0));CHKERRQ(ierr);
+      ierr = BVScaleColumn(svd->V,j,1.0/PETSC_SQRT2);CHKERRQ(ierr);
       ierr = VecResetArray(x1);CHKERRQ(ierr);
       ierr = VecResetArray(x2);CHKERRQ(ierr);
       ierr = VecRestoreArrayRead(x,&px);CHKERRQ(ierr);
       j++;
     }
   }
-  svd->nconv = j;
-
   ierr = VecDestroy(&x);CHKERRQ(ierr);
   ierr = VecDestroy(&x1);CHKERRQ(ierr);
   ierr = VecDestroy(&x2);CHKERRQ(ierr);
@@ -509,13 +528,14 @@ SLEPC_EXTERN PetscErrorCode SVDCreate_Cyclic(SVD svd)
 
   PetscFunctionBegin;
   ierr = PetscNewLog(svd,&cyclic);CHKERRQ(ierr);
-  svd->data                      = (void*)cyclic;
-  svd->ops->solve                = SVDSolve_Cyclic;
-  svd->ops->setup                = SVDSetUp_Cyclic;
-  svd->ops->setfromoptions       = SVDSetFromOptions_Cyclic;
-  svd->ops->destroy              = SVDDestroy_Cyclic;
-  svd->ops->reset                = SVDReset_Cyclic;
-  svd->ops->view                 = SVDView_Cyclic;
+  svd->data                = (void*)cyclic;
+  svd->ops->solve          = SVDSolve_Cyclic;
+  svd->ops->setup          = SVDSetUp_Cyclic;
+  svd->ops->setfromoptions = SVDSetFromOptions_Cyclic;
+  svd->ops->destroy        = SVDDestroy_Cyclic;
+  svd->ops->reset          = SVDReset_Cyclic;
+  svd->ops->view           = SVDView_Cyclic;
+  svd->ops->computevectors = SVDComputeVectors_Cyclic;
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDCyclicSetEPS_C",SVDCyclicSetEPS_Cyclic);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDCyclicGetEPS_C",SVDCyclicGetEPS_Cyclic);CHKERRQ(ierr);
   ierr = PetscObjectComposeFunction((PetscObject)svd,"SVDCyclicSetExplicitMatrix_C",SVDCyclicSetExplicitMatrix_Cyclic);CHKERRQ(ierr);
