@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2020, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -105,9 +105,9 @@ static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rno
 {
   PetscErrorCode ierr;
   PetscInt       i;
-  PetscBLASInt   mm=1,mout,info,ld,n,*select,inc = 1;
-  PetscScalar    tmp,done=1.0,zero=0.0;
-  PetscReal      norm;
+  PetscBLASInt   mm=1,mout,info,ld,n,*select,inc=1,cols=1,zero=0;
+  PetscScalar    sone=1.0,szero=0.0;
+  PetscReal      norm,done=1.0;
   PetscBool      iscomplex = PETSC_FALSE;
   PetscScalar    *A = ds->mat[DS_MAT_A];
   PetscScalar    *Q = ds->mat[DS_MAT_Q];
@@ -140,22 +140,20 @@ static PetscErrorCode DSVectors_NHEP_Eigen_Some(DS ds,PetscInt *k,PetscReal *rno
   /* accumulate and normalize eigenvectors */
   if (ds->state>=DS_STATE_CONDENSED) {
     ierr = PetscArraycpy(ds->work,Y,mout*ld);CHKERRQ(ierr);
-    PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&done,Q,&ld,ds->work,&inc,&zero,Y,&inc));
+    PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&sone,Q,&ld,ds->work,&inc,&szero,Y,&inc));
 #if !defined(PETSC_USE_COMPLEX)
-    if (iscomplex) PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&done,Q,&ld,ds->work+ld,&inc,&zero,Y+ld,&inc));
+    if (iscomplex) PetscStackCallBLAS("BLASgemv",BLASgemv_("N",&n,&n,&sone,Q,&ld,ds->work+ld,&inc,&szero,Y+ld,&inc));
 #endif
+    cols = 1;
     norm = BLASnrm2_(&n,Y,&inc);
 #if !defined(PETSC_USE_COMPLEX)
     if (iscomplex) {
-      tmp = BLASnrm2_(&n,Y+ld,&inc);
-      norm = SlepcAbsEigenvalue(norm,tmp);
+      norm = SlepcAbsEigenvalue(norm,BLASnrm2_(&n,Y+ld,&inc));
+      cols = 2;
     }
 #endif
-    tmp = 1.0 / norm;
-    PetscStackCallBLAS("BLASscal",BLASscal_(&n,&tmp,Y,&inc));
-#if !defined(PETSC_USE_COMPLEX)
-    if (iscomplex) PetscStackCallBLAS("BLASscal",BLASscal_(&n,&tmp,Y+ld,&inc));
-#endif
+    PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&norm,&done,&n,&cols,Y,&ld,&info));
+    SlepcCheckLapackInfo("lascl",info);
   }
 
   /* set output arguments */
@@ -171,10 +169,10 @@ static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
 {
   PetscErrorCode ierr;
   PetscInt       i;
-  PetscBLASInt   n,ld,mout,info,inc = 1;
+  PetscBLASInt   n,ld,mout,info,inc=1,cols,zero=0;
   PetscBool      iscomplex;
-  PetscScalar    *X,*Y,*Z,*A = ds->mat[DS_MAT_A],tmp;
-  PetscReal      norm;
+  PetscScalar    *X,*Y,*Z,*A = ds->mat[DS_MAT_A];
+  PetscReal      norm,done=1.0;
   const char     *side,*back;
 
   PetscFunctionBegin;
@@ -207,18 +205,16 @@ static PetscErrorCode DSVectors_NHEP_Eigen_All(DS ds,PetscBool left)
   /* normalize eigenvectors */
   for (i=0;i<n;i++) {
     iscomplex = (i<n-1 && A[i+1+i*ld]!=0.0)? PETSC_TRUE: PETSC_FALSE;
+    cols = 1;
     norm = BLASnrm2_(&n,Z+i*ld,&inc);
 #if !defined(PETSC_USE_COMPLEX)
     if (iscomplex) {
-      tmp = BLASnrm2_(&n,Z+(i+1)*ld,&inc);
-      norm = SlepcAbsEigenvalue(norm,tmp);
+      norm = SlepcAbsEigenvalue(norm,BLASnrm2_(&n,Z+(i+1)*ld,&inc));
+      cols = 2;
     }
 #endif
-    tmp = 1.0 / norm;
-    PetscStackCallBLAS("BLASscal",BLASscal_(&n,&tmp,Z+i*ld,&inc));
-#if !defined(PETSC_USE_COMPLEX)
-    if (iscomplex) PetscStackCallBLAS("BLASscal",BLASscal_(&n,&tmp,Z+(i+1)*ld,&inc));
-#endif
+    PetscStackCallBLAS("LAPACKlascl",LAPACKlascl_("G",&zero,&zero,&norm,&done,&n,&cols,Z+i*ld,&ld,&info));
+    SlepcCheckLapackInfo("lascl",info);
     if (iscomplex) i++;
   }
   PetscFunctionReturn(0);
@@ -257,13 +253,9 @@ PetscErrorCode DSVectors_NHEP(DS ds,DSMatType mat,PetscInt *j,PetscReal *rnorm)
     case DS_MAT_U:
     case DS_MAT_VT:
       SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_SUP,"Not implemented yet");
-      break;
     default:
       SETERRQ(PetscObjectComm((PetscObject)ds),PETSC_ERR_ARG_OUTOFRANGE,"Invalid mat parameter");
   }
-  if (ds->state < DS_STATE_CONDENSED) {
-    ierr = DSSetState(ds,DS_STATE_CONDENSED);CHKERRQ(ierr);
-  }
   PetscFunctionReturn(0);
 }
 
@@ -310,95 +302,13 @@ static PetscErrorCode DSSort_NHEP_Arbitrary(DS ds,PetscScalar *wr,PetscScalar *w
   PetscFunctionReturn(0);
 }
 
-static PetscErrorCode DSSort_NHEP_Total(DS ds,PetscScalar *wr,PetscScalar *wi)
-{
-  PetscErrorCode ierr;
-  PetscScalar    re;
-  PetscInt       i,j,pos,result;
-  PetscBLASInt   ifst,ilst,info,n,ld;
-  PetscScalar    *T = ds->mat[DS_MAT_A];
-  PetscScalar    *Q = ds->mat[DS_MAT_Q];
-#if !defined(PETSC_USE_COMPLEX)
-  PetscScalar    *work,im;
-#endif
-
-  PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-  ierr = DSAllocateWork_Private(ds,ld,0,0);CHKERRQ(ierr);
-  work = ds->work;
-#endif
-  /* selection sort */
-  for (i=ds->l;i<n-1;i++) {
-    re = wr[i];
-#if !defined(PETSC_USE_COMPLEX)
-    im = wi[i];
-#endif
-    pos = 0;
-    j=i+1; /* j points to the next eigenvalue */
-#if !defined(PETSC_USE_COMPLEX)
-    if (im != 0) j=i+2;
-#endif
-    /* find minimum eigenvalue */
-    for (;j<n;j++) {
-#if !defined(PETSC_USE_COMPLEX)
-      ierr = SlepcSCCompare(ds->sc,re,im,wr[j],wi[j],&result);CHKERRQ(ierr);
-#else
-      ierr = SlepcSCCompare(ds->sc,re,0.0,wr[j],0.0,&result);CHKERRQ(ierr);
-#endif
-      if (result > 0) {
-        re = wr[j];
-#if !defined(PETSC_USE_COMPLEX)
-        im = wi[j];
-#endif
-        pos = j;
-      }
-#if !defined(PETSC_USE_COMPLEX)
-      if (wi[j] != 0) j++;
-#endif
-    }
-    if (pos) {
-      /* interchange blocks */
-      ierr = PetscBLASIntCast(pos+1,&ifst);CHKERRQ(ierr);
-      ierr = PetscBLASIntCast(i+1,&ilst);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-      PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,work,&info));
-#else
-      PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,&info));
-#endif
-      SlepcCheckLapackInfo("trexc",info);
-      /* recover original eigenvalues from T matrix */
-      for (j=i;j<n;j++) {
-        wr[j] = T[j+j*ld];
-#if !defined(PETSC_USE_COMPLEX)
-        if (j<n-1 && T[j+1+j*ld] != 0.0) {
-          /* complex conjugate eigenvalue */
-          wi[j] = PetscSqrtReal(PetscAbsReal(T[j+1+j*ld])) *
-                  PetscSqrtReal(PetscAbsReal(T[j+(j+1)*ld]));
-          wr[j+1] = wr[j];
-          wi[j+1] = -wi[j];
-          j++;
-        } else {
-          wi[j] = 0.0;
-        }
-#endif
-      }
-    }
-#if !defined(PETSC_USE_COMPLEX)
-    if (wi[i] != 0) i++;
-#endif
-  }
-  PetscFunctionReturn(0);
-}
-
 PetscErrorCode DSSort_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr,PetscScalar *ri,PetscInt *k)
 {
   PetscErrorCode ierr;
 
   PetscFunctionBegin;
   if (!rr || wr == rr) {
-    ierr = DSSort_NHEP_Total(ds,wr,wi);CHKERRQ(ierr);
+    ierr = DSSort_NHEP_Total(ds,ds->mat[DS_MAT_A],ds->mat[DS_MAT_Q],wr,wi);CHKERRQ(ierr);
   } else {
     ierr = DSSort_NHEP_Arbitrary(ds,wr,wi,rr,ri,k);CHKERRQ(ierr);
   }
@@ -408,64 +318,9 @@ PetscErrorCode DSSort_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi,PetscScalar *rr
 static PetscErrorCode DSSortWithPermutation_NHEP(DS ds,PetscInt *perm,PetscScalar *wr,PetscScalar *wi)
 {
   PetscErrorCode ierr;
-  PetscInt       i,j,pos,inc=1;
-  PetscBLASInt   ifst,ilst,info,n,ld;
-  PetscScalar    *T = ds->mat[DS_MAT_A];
-  PetscScalar    *Q = ds->mat[DS_MAT_Q];
-#if !defined(PETSC_USE_COMPLEX)
-  PetscScalar    *work;
-#endif
 
   PetscFunctionBegin;
-  ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-  ierr = DSAllocateWork_Private(ds,ld,0,0);CHKERRQ(ierr);
-  work = ds->work;
-#endif
-  for (i=ds->l;i<n-1;i++) {
-    pos = perm[i];
-#if !defined(PETSC_USE_COMPLEX)
-    inc = (pos<n-1 && T[pos+1+pos*ld] != 0.0)? 2: 1;
-#endif
-    if (pos!=i) {
-#if !defined(PETSC_USE_COMPLEX)
-      if ((T[pos+(pos-1)*ld] != 0.0 && perm[i+1]!=pos-1) || (T[pos+1+pos*ld] != 0.0 && perm[i+1]!=pos+1))
- SETERRQ1(PETSC_COMM_SELF,1,"Invalid permutation due to a 2x2 block at position %D",pos);
-#endif
-
-      /* interchange blocks */
-      ierr = PetscBLASIntCast(pos+1,&ifst);CHKERRQ(ierr);
-      ierr = PetscBLASIntCast(i+1,&ilst);CHKERRQ(ierr);
-#if !defined(PETSC_USE_COMPLEX)
-      PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,work,&info));
-#else
-      PetscStackCallBLAS("LAPACKtrexc",LAPACKtrexc_("V",&n,T,&ld,Q,&ld,&ifst,&ilst,&info));
-#endif
-      SlepcCheckLapackInfo("trexc",info);
-      for (j=i+1;j<n;j++) {
-        if (perm[j]>=i && perm[j]<pos) perm[j]+=inc;
-      }
-      perm[i] = i;
-      if (inc==2) perm[i+1] = i+1;
-    }
-    if (inc==2) i++;
-  }
-  /* recover original eigenvalues from T matrix */
-  for (j=ds->l;j<n;j++) {
-    wr[j] = T[j+j*ld];
-#if !defined(PETSC_USE_COMPLEX)
-    if (j<n-1 && T[j+1+j*ld] != 0.0) {
-      /* complex conjugate eigenvalue */
-      wi[j] = PetscSqrtReal(PetscAbsReal(T[j+1+j*ld])) * PetscSqrtReal(PetscAbsReal(T[j+(j+1)*ld]));
-      wr[j+1] = wr[j];
-      wi[j+1] = -wi[j];
-      j++;
-    } else {
-      wi[j] = 0.0;
-    }
-#endif
-  }
+  ierr = DSSortWithPermutation_NHEP_Private(ds,perm,ds->mat[DS_MAT_A],ds->mat[DS_MAT_Q],wr,wi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -491,126 +346,15 @@ PetscErrorCode DSUpdateExtraRow_NHEP(DS ds)
   PetscFunctionReturn(0);
 }
 
-/*
-   Reduce a matrix A to upper Hessenberg form Q'*A*Q, where Q is an orthogonal matrix.
-   The result overwrites A. Matrix A has the form
-
-         [ S | * ]
-     A = [-------]
-         [ R | H ]
-
-  where S is an upper (quasi-)triangular matrix of order k, H is an upper Hessenberg
-  matrix of order n-k, and R is all zeros except the first row (the arrow).
-  The algorithm uses elementary reflectors to annihilate entries in the arrow, and
-  then proceeds upwards.
-  If ilo>1, then it is assumed that the first ilo-1 entries of the arrow are zero, and
-  hence the first ilo-1 rows and columns of Q are set to the identity matrix.
-
-  Required workspace is 2*n.
-*/
-static PetscErrorCode ArrowHessenberg(PetscBLASInt n,PetscBLASInt k,PetscBLASInt ilo,PetscScalar *A,PetscBLASInt lda,PetscScalar *Q,PetscBLASInt ldq,PetscScalar *work)
-{
-  PetscBLASInt i,j,n0,m,inc=1,incn=-1;
-  PetscScalar  t,*v=work,*w=work+n,tau,tauc;
-
-  PetscFunctionBegin;
-  m = n-ilo+1;
-  for (i=k;i>ilo;i--) {
-    for (j=0;j<=i-ilo;j++) v[j] = A[i+(i-j-1)*lda];  /* _larfg does not allow negative inc, so use buffer */
-    n0 = i-ilo+1;
-    PetscStackCallBLAS("LAPACKlarfg",LAPACKlarfg_(&n0,v,v+1,&inc,&tau));
-    for (j=1;j<=i-ilo;j++) v[j] = PetscConj(v[j]);
-    tauc = PetscConj(tau);
-    A[i+(i-1)*lda] = v[0];
-    v[0] = 1.0;
-    PetscStackCallBLAS("LAPACKlarf",LAPACKlarf_("R",&i,&n0,v,&incn,&tauc,A+(ilo-1)*lda,&lda,w));
-    for (j=1;j<=i-ilo;j++) A[i+(i-j-1)*lda] = 0.0;
-    PetscStackCallBLAS("LAPACKlarf",LAPACKlarf_("L",&n0,&m,v,&incn,&tau,A+ilo-1+(ilo-1)*lda,&lda,w));
-    PetscStackCallBLAS("LAPACKlarf",LAPACKlarf_("L",&n0,&m,v,&incn,&tau,Q+ilo-1+(ilo-1)*ldq,&ldq,w));
-  }
-  /* trivial in-place transposition of Q */
-  for (j=ilo-1;j<k;j++) {
-    for (i=j;i<k;i++) {
-      t = Q[i+j*ldq];
-      if (i!=j) Q[i+j*ldq] = PetscConj(Q[j+i*ldq]);
-      Q[j+i*ldq] = PetscConj(t);
-    }
-  }
-  PetscFunctionReturn(0);
-}
-
 PetscErrorCode DSSolve_NHEP(DS ds,PetscScalar *wr,PetscScalar *wi)
 {
   PetscErrorCode ierr;
-  PetscScalar    *work,*tau;
-  PetscInt       i,j;
-  PetscBLASInt   ilo,lwork,info,n,k,ld;
-  PetscScalar    *A = ds->mat[DS_MAT_A];
-  PetscScalar    *Q = ds->mat[DS_MAT_Q];
 
   PetscFunctionBegin;
 #if !defined(PETSC_USE_COMPLEX)
   PetscValidScalarPointer(wi,3);
 #endif
-  ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->ld,&ld);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->l+1,&ilo);CHKERRQ(ierr);
-  ierr = PetscBLASIntCast(ds->k,&k);CHKERRQ(ierr);
-  ierr = DSAllocateWork_Private(ds,ld+6*ld,0,0);CHKERRQ(ierr);
-  tau  = ds->work;
-  work = ds->work+ld;
-  lwork = 6*ld;
-
-  /* initialize orthogonal matrix */
-  ierr = PetscArrayzero(Q,ld*ld);CHKERRQ(ierr);
-  for (i=0;i<n;i++) Q[i+i*ld] = 1.0;
-  if (n==1) { /* quick return */
-    wr[0] = A[0];
-    wi[0] = 0.0;
-    PetscFunctionReturn(0);
-  }
-
-  /* reduce to upper Hessenberg form */
-  if (ds->state<DS_STATE_INTERMEDIATE) {
-    if (PETSC_FALSE && k>0) {
-      ierr = ArrowHessenberg(n,k,ilo,A,ld,Q,ld,work);CHKERRQ(ierr);
-    } else {
-      PetscStackCallBLAS("LAPACKgehrd",LAPACKgehrd_(&n,&ilo,&n,A,&ld,tau,work,&lwork,&info));
-      SlepcCheckLapackInfo("gehrd",info);
-      for (j=0;j<n-1;j++) {
-        for (i=j+2;i<n;i++) {
-          Q[i+j*ld] = A[i+j*ld];
-          A[i+j*ld] = 0.0;
-        }
-      }
-      PetscStackCallBLAS("LAPACKorghr",LAPACKorghr_(&n,&ilo,&n,Q,&ld,tau,work,&lwork,&info));
-      SlepcCheckLapackInfo("orghr",info);
-    }
-  }
-
-  /* compute the (real) Schur form */
-#if !defined(PETSC_USE_COMPLEX)
-  PetscStackCallBLAS("LAPACKhseqr",LAPACKhseqr_("S","V",&n,&ilo,&n,A,&ld,wr,wi,Q,&ld,work,&lwork,&info));
-  for (j=0;j<ds->l;j++) {
-    if (j==n-1 || A[j+1+j*ld] == 0.0) {
-      /* real eigenvalue */
-      wr[j] = A[j+j*ld];
-      wi[j] = 0.0;
-    } else {
-      /* complex eigenvalue */
-      wr[j] = A[j+j*ld];
-      wr[j+1] = A[j+j*ld];
-      wi[j] = PetscSqrtReal(PetscAbsReal(A[j+1+j*ld])) *
-              PetscSqrtReal(PetscAbsReal(A[j+(j+1)*ld]));
-      wi[j+1] = -wi[j];
-      j++;
-    }
-  }
-#else
-  PetscStackCallBLAS("LAPACKhseqr",LAPACKhseqr_("S","V",&n,&ilo,&n,A,&ld,wr,Q,&ld,work,&lwork,&info));
-  if (wi) for (i=ds->l;i<n;i++) wi[i] = 0.0;
-#endif
-  SlepcCheckLapackInfo("hseqr",info);
+  ierr = DSSolve_NHEP_Private(ds,ds->mat[DS_MAT_A],ds->mat[DS_MAT_Q],wr,wi);CHKERRQ(ierr);
   PetscFunctionReturn(0);
 }
 
@@ -629,56 +373,63 @@ PetscErrorCode DSSynchronize_NHEP(DS ds,PetscScalar eigr[],PetscScalar eigi[])
   ierr = PetscMPIIntCast(k*sizeof(PetscScalar),&size);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(ds->n-l,&n);CHKERRQ(ierr);
   ierr = PetscMPIIntCast(ld*(ds->n-l),&ldn);CHKERRQ(ierr);
-  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ds),&rank);CHKERRQ(ierr);
+  ierr = MPI_Comm_rank(PetscObjectComm((PetscObject)ds),&rank);CHKERRMPI(ierr);
   if (!rank) {
-    ierr = MPI_Pack(ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    ierr = MPI_Pack(ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     if (ds->state>DS_STATE_RAW) {
-      ierr = MPI_Pack(ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      ierr = MPI_Pack(ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     }
     if (eigr) {
-      ierr = MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      ierr = MPI_Pack(eigr+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     }
     if (eigi) {
-      ierr = MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      ierr = MPI_Pack(eigi+l,n,MPIU_SCALAR,ds->work,size,&off,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     }
   }
-  ierr = MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+  ierr = MPI_Bcast(ds->work,size,MPI_BYTE,0,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
   if (rank) {
-    ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+    ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_A]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     if (ds->state>DS_STATE_RAW) {
-      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      ierr = MPI_Unpack(ds->work,size,&off,ds->mat[DS_MAT_Q]+l*ld,ldn,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     }
     if (eigr) {
-      ierr = MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      ierr = MPI_Unpack(ds->work,size,&off,eigr+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     }
     if (eigi) {
-      ierr = MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRQ(ierr);
+      ierr = MPI_Unpack(ds->work,size,&off,eigi+l,n,MPIU_SCALAR,PetscObjectComm((PetscObject)ds));CHKERRMPI(ierr);
     }
   }
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode DSTruncate_NHEP(DS ds,PetscInt n)
+PetscErrorCode DSTruncate_NHEP(DS ds,PetscInt n,PetscBool trim)
 {
-  PetscInt    i,newn,ld=ds->ld,l=ds->l;
-  PetscScalar *A;
+  PetscInt    i,ld=ds->ld,l=ds->l;
+  PetscScalar *A = ds->mat[DS_MAT_A];
 
   PetscFunctionBegin;
-  if (ds->state==DS_STATE_CONDENSED) ds->t = ds->n;
-  A = ds->mat[DS_MAT_A];
-  /* be careful not to break a diagonal 2x2 block */
-  if (A[n+(n-1)*ld]==0.0) newn = n;
-  else {
-    if (n<ds->n-1) newn = n+1;
-    else newn = n-1;
+#if defined(PETSC_USE_DEBUG)
+  /* make sure diagonal 2x2 block is not broken */
+  if (ds->state>=DS_STATE_CONDENSED && n>0 && n<ds->n && A[n+(n-1)*ld]!=0.0) SETERRQ(PETSC_COMM_SELF,PETSC_ERR_ARG_WRONG,"The given size would break a 2x2 block, call DSGetTruncateSize() first");
+#endif
+  if (trim) {
+    if (ds->extrarow) {   /* clean extra row */
+      for (i=l;i<ds->n;i++) A[ds->n+i*ld] = 0.0;
+    }
+    ds->l = 0;
+    ds->k = 0;
+    ds->n = n;
+    ds->t = ds->n;   /* truncated length equal to the new dimension */
+  } else {
+    if (ds->extrarow && ds->k==ds->n) {
+      /* copy entries of extra row to the new position, then clean last row */
+      for (i=l;i<n;i++) A[n+i*ld] = A[ds->n+i*ld];
+      for (i=l;i<ds->n;i++) A[ds->n+i*ld] = 0.0;
+    }
+    ds->k = (ds->extrarow)? n: 0;
+    ds->t = ds->n;   /* truncated length equal to previous dimension */
+    ds->n = n;
   }
-  if (ds->extrarow && ds->k==ds->n) {
-    /* copy entries of extra row to the new position, then clean last row */
-    for (i=l;i<newn;i++) A[newn+i*ld] = A[ds->n+i*ld];
-    for (i=l;i<ds->n;i++) A[ds->n+i*ld] = 0.0;
-  }
-  ds->k = 0;
-  ds->n = newn;
   PetscFunctionReturn(0);
 }
 
@@ -721,14 +472,14 @@ PetscErrorCode DSCond_NHEP(DS ds,PetscReal *cond)
   PetscFunctionReturn(0);
 }
 
-PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,PetscBool recover,PetscScalar *gin,PetscReal *gamma)
+PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,PetscBool recover,PetscScalar *gin,PetscReal *gammaout)
 {
   PetscErrorCode ierr;
   PetscInt       i,j;
   PetscBLASInt   *ipiv,info,n,ld,one=1,ncol;
   PetscScalar    *A,*B,*Q,*g=gin,*ghat;
   PetscScalar    done=1.0,dmone=-1.0,dzero=0.0;
-  PetscReal      gnorm;
+  PetscReal      gamma=1.0;
 
   PetscFunctionBegin;
   ierr = PetscBLASIntCast(ds->n,&n);CHKERRQ(ierr);
@@ -784,10 +535,10 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
   }
 
   /* Compute gamma factor */
-  if (gamma) {
-    gnorm = 0.0;
-    for (i=0;i<n;i++) gnorm = gnorm + PetscRealPart(g[i]*PetscConj(g[i]));
-    *gamma = PetscSqrtReal(1.0+gnorm);
+  if (gammaout || (recover && ds->extrarow)) gamma = SlepcAbs(1.0,BLASnrm2_(&n,g,&one));
+  if (gammaout) *gammaout = gamma;
+  if (recover && ds->extrarow) {
+    for (j=ds->l;j<ds->l+ds->k;j++) A[ds->n+j*ld] *= gamma;
   }
   PetscFunctionReturn(0);
 }
@@ -795,17 +546,18 @@ PetscErrorCode DSTranslateHarmonic_NHEP(DS ds,PetscScalar tau,PetscReal beta,Pet
 SLEPC_EXTERN PetscErrorCode DSCreate_NHEP(DS ds)
 {
   PetscFunctionBegin;
-  ds->ops->allocate      = DSAllocate_NHEP;
-  ds->ops->view          = DSView_NHEP;
-  ds->ops->vectors       = DSVectors_NHEP;
-  ds->ops->solve[0]      = DSSolve_NHEP;
-  ds->ops->sort          = DSSort_NHEP;
-  ds->ops->sortperm      = DSSortWithPermutation_NHEP;
-  ds->ops->synchronize   = DSSynchronize_NHEP;
-  ds->ops->truncate      = DSTruncate_NHEP;
-  ds->ops->update        = DSUpdateExtraRow_NHEP;
-  ds->ops->cond          = DSCond_NHEP;
-  ds->ops->transharm     = DSTranslateHarmonic_NHEP;
+  ds->ops->allocate        = DSAllocate_NHEP;
+  ds->ops->view            = DSView_NHEP;
+  ds->ops->vectors         = DSVectors_NHEP;
+  ds->ops->solve[0]        = DSSolve_NHEP;
+  ds->ops->sort            = DSSort_NHEP;
+  ds->ops->sortperm        = DSSortWithPermutation_NHEP;
+  ds->ops->synchronize     = DSSynchronize_NHEP;
+  ds->ops->gettruncatesize = DSGetTruncateSize_Default;
+  ds->ops->truncate        = DSTruncate_NHEP;
+  ds->ops->update          = DSUpdateExtraRow_NHEP;
+  ds->ops->cond            = DSCond_NHEP;
+  ds->ops->transharm       = DSTranslateHarmonic_NHEP;
   PetscFunctionReturn(0);
 }
 
