@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -12,6 +12,7 @@
 */
 
 #include <slepc/private/rgimpl.h>      /*I "slepcrg.h" I*/
+#include <petscdraw.h>
 
 #define VERTMAX 30
 
@@ -20,6 +21,8 @@ typedef struct {
   PetscScalar *vr,*vi;   /* array of vertices (vi not used in complex scalars) */
 } RG_POLYGON;
 
+PetscErrorCode RGComputeBoundingBox_Polygon(RG,PetscReal*,PetscReal*,PetscReal*,PetscReal*);
+
 #if !defined(PETSC_USE_COMPLEX)
 static PetscBool CheckSymmetry(PetscInt n,PetscScalar *vr,PetscScalar *vi)
 {
@@ -78,7 +81,7 @@ static PetscErrorCode RGPolygonSetVertices_Polygon(RG rg,PetscInt n,PetscScalar
 /*@
    RGPolygonSetVertices - Sets the vertices that define the polygon region.
 
-   Logically Collective on RG
+   Logically Collective on rg
 
    Input Parameters:
 +  rg - the region context
@@ -112,9 +115,9 @@ PetscErrorCode RGPolygonSetVertices(RG rg,PetscInt n,PetscScalar vr[],PetscScala
   PetscFunctionBegin;
   PetscValidHeaderSpecific(rg,RG_CLASSID,1);
   PetscValidLogicalCollectiveInt(rg,n,2);
-  PetscValidPointer(vr,3);
+  PetscValidScalarPointer(vr,3);
 #if !defined(PETSC_USE_COMPLEX)
-  PetscValidPointer(vi,4);
+  PetscValidScalarPointer(vi,4);
 #endif
   ierr = PetscTryMethod(rg,"RGPolygonSetVertices_C",(RG,PetscInt,PetscScalar*,PetscScalar*),(rg,n,vr,vi));CHKERRQ(ierr);
   PetscFunctionReturn(0);
@@ -122,16 +125,32 @@ PetscErrorCode RGPolygonSetVertices(RG rg,PetscInt n,PetscScalar vr[],PetscScala
 
 static PetscErrorCode RGPolygonGetVertices_Polygon(RG rg,PetscInt *n,PetscScalar **vr,PetscScalar **vi)
 {
-  RG_POLYGON *ctx = (RG_POLYGON*)rg->data;
+  PetscErrorCode ierr;
+  RG_POLYGON     *ctx = (RG_POLYGON*)rg->data;
+  PetscInt       i;
 
   PetscFunctionBegin;
-  if (n)  *n  = ctx->n;
-  if (vr) *vr = ctx->vr;
-  if (vi) *vi = ctx->vi;
+  if (n) *n  = ctx->n;
+  if (vr) {
+    if (!ctx->n) *vr = NULL;
+    else {
+      ierr = PetscMalloc1(ctx->n,vr);CHKERRQ(ierr);
+      for (i=0;i<ctx->n;i++) (*vr)[i] = ctx->vr[i];
+    }
+  }
+#if !defined(PETSC_USE_COMPLEX)
+  if (vi) {
+    if (!ctx->n) *vi = NULL;
+    else {
+      ierr = PetscMalloc1(ctx->n,vi);CHKERRQ(ierr);
+      for (i=0;i<ctx->n;i++) (*vi)[i] = ctx->vi[i];
+    }
+  }
+#endif
   PetscFunctionReturn(0);
 }
 
-/*@
+/*@C
    RGPolygonGetVertices - Gets the vertices that define the polygon region.
 
    Not Collective
@@ -145,7 +164,9 @@ static PetscErrorCode RGPolygonGetVertices_Polygon(RG rg,PetscInt *n,PetscScalar
 -  vi - array of vertices (imaginary part)
 
    Notes:
-   The returned arrays must NOT be freed by the calling application.
+   The values passed by user with RGPolygonSetVertices() are returned (or null
+   pointers otherwise).
+   The returned arrays should be freed by the user when no longer needed.
 
    Level: advanced
 
@@ -165,29 +186,76 @@ PetscErrorCode RGView_Polygon(RG rg,PetscViewer viewer)
 {
   PetscErrorCode ierr;
   RG_POLYGON     *ctx = (RG_POLYGON*)rg->data;
-  PetscBool      isascii;
+  PetscBool      isdraw,isascii;
+  int            winw,winh;
+  PetscDraw      draw;
+  PetscDrawAxis  axis;
+  PetscReal      a,b,c,d,ab,cd,lx,ly,w,x0,y0,x1,y1,scale=1.2;
   PetscInt       i;
   char           str[50];
 
   PetscFunctionBegin;
+  ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERDRAW,&isdraw);CHKERRQ(ierr);
   ierr = PetscObjectTypeCompare((PetscObject)viewer,PETSCVIEWERASCII,&isascii);CHKERRQ(ierr);
   if (isascii) {
     ierr = PetscViewerASCIIPrintf(viewer,"  vertices: ");CHKERRQ(ierr);
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_FALSE);CHKERRQ(ierr);
     for (i=0;i<ctx->n;i++) {
 #if defined(PETSC_USE_COMPLEX)
-      ierr = SlepcSNPrintfScalar(str,50,ctx->vr[i],PETSC_FALSE);CHKERRQ(ierr);
+      ierr = SlepcSNPrintfScalar(str,sizeof(str),ctx->vr[i],PETSC_FALSE);CHKERRQ(ierr);
 #else
       if (ctx->vi[i]!=0.0) {
-        ierr = PetscSNPrintf(str,50,"%g%+gi",(double)ctx->vr[i],(double)ctx->vi[i]);CHKERRQ(ierr);
+        ierr = PetscSNPrintf(str,sizeof(str),"%g%+gi",(double)ctx->vr[i],(double)ctx->vi[i]);CHKERRQ(ierr);
       } else {
-        ierr = PetscSNPrintf(str,50,"%g",(double)ctx->vr[i]);CHKERRQ(ierr);
+        ierr = PetscSNPrintf(str,sizeof(str),"%g",(double)ctx->vr[i]);CHKERRQ(ierr);
       }
 #endif
       ierr = PetscViewerASCIIPrintf(viewer,"%s%s",str,(i<ctx->n-1)?", ":"");CHKERRQ(ierr);
     }
     ierr = PetscViewerASCIIPrintf(viewer,"\n");CHKERRQ(ierr);
     ierr = PetscViewerASCIIUseTabs(viewer,PETSC_TRUE);CHKERRQ(ierr);
+  } else if (isdraw) {
+    ierr = PetscViewerDrawGetDraw(viewer,0,&draw);CHKERRQ(ierr);
+    ierr = PetscDrawCheckResizedWindow(draw);CHKERRQ(ierr);
+    ierr = PetscDrawGetWindowSize(draw,&winw,&winh);CHKERRQ(ierr);
+    winw = PetscMax(winw,1); winh = PetscMax(winh,1);
+    ierr = PetscDrawClear(draw);CHKERRQ(ierr);
+    ierr = PetscDrawSetTitle(draw,"Polygonal region");CHKERRQ(ierr);
+    ierr = PetscDrawAxisCreate(draw,&axis);CHKERRQ(ierr);
+    ierr = RGComputeBoundingBox_Polygon(rg,&a,&b,&c,&d);CHKERRQ(ierr);
+    a *= rg->sfactor;
+    b *= rg->sfactor;
+    c *= rg->sfactor;
+    d *= rg->sfactor;
+    lx = b-a;
+    ly = d-c;
+    ab = (a+b)/2;
+    cd = (c+d)/2;
+    w  = scale*PetscMax(lx/winw,ly/winh)/2;
+    ierr = PetscDrawAxisSetLimits(axis,ab-w*winw,ab+w*winw,cd-w*winh,cd+w*winh);CHKERRQ(ierr);
+    ierr = PetscDrawAxisDraw(axis);CHKERRQ(ierr);
+    ierr = PetscDrawAxisDestroy(&axis);CHKERRQ(ierr);
+    for (i=0;i<ctx->n;i++) {
+#if defined(PETSC_USE_COMPLEX)
+      x0 = PetscRealPart(ctx->vr[i]); y0 = PetscImaginaryPart(ctx->vr[i]);
+      if (i<ctx->n-1) {
+        x1 = PetscRealPart(ctx->vr[i+1]); y1 = PetscImaginaryPart(ctx->vr[i+1]);
+      } else {
+        x1 = PetscRealPart(ctx->vr[0]); y1 = PetscImaginaryPart(ctx->vr[0]);
+      }
+#else
+      x0 = ctx->vr[i]; y0 = ctx->vi[i];
+      if (i<ctx->n-1) {
+        x1 = ctx->vr[i+1]; y1 = ctx->vi[i+1];
+      } else {
+        x1 = ctx->vr[0]; y1 = ctx->vi[0];
+      }
+#endif
+      ierr = PetscDrawLine(draw,x0*rg->sfactor,y0*rg->sfactor,x1*rg->sfactor,y1*rg->sfactor,PETSC_DRAW_MAGENTA);CHKERRQ(ierr);
+    }
+    ierr = PetscDrawFlush(draw);CHKERRQ(ierr);
+    ierr = PetscDrawSave(draw);CHKERRQ(ierr);
+    ierr = PetscDrawPause(draw);CHKERRQ(ierr);
   }
   PetscFunctionReturn(0);
 }
@@ -380,7 +448,7 @@ PetscErrorCode RGDestroy_Polygon(RG rg)
   PetscFunctionReturn(0);
 }
 
-PETSC_EXTERN PetscErrorCode RGCreate_Polygon(RG rg)
+SLEPC_EXTERN PetscErrorCode RGCreate_Polygon(RG rg)
 {
   RG_POLYGON     *polygon;
   PetscErrorCode ierr;
