@@ -1,7 +1,7 @@
 /*
    - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    SLEPc - Scalable Library for Eigenvalue Problem Computations
-   Copyright (c) 2002-2017, Universitat Politecnica de Valencia, Spain
+   Copyright (c) 2002-2021, Universitat Politecnica de Valencia, Spain
 
    This file is part of SLEPc.
    SLEPc is distributed under a 2-clause BSD license (see LICENSE).
@@ -10,10 +10,10 @@
 
 #include <slepc/private/rgimpl.h>      /*I "slepcrg.h" I*/
 
-PETSC_EXTERN PetscErrorCode RGCreate_Interval(RG);
-PETSC_EXTERN PetscErrorCode RGCreate_Ellipse(RG);
-PETSC_EXTERN PetscErrorCode RGCreate_Ring(RG);
-PETSC_EXTERN PetscErrorCode RGCreate_Polygon(RG);
+SLEPC_EXTERN PetscErrorCode RGCreate_Interval(RG);
+SLEPC_EXTERN PetscErrorCode RGCreate_Ellipse(RG);
+SLEPC_EXTERN PetscErrorCode RGCreate_Ring(RG);
+SLEPC_EXTERN PetscErrorCode RGCreate_Polygon(RG);
 
 /*@C
    RGRegisterAll - Registers all of the regions in the RG package.
